#!/usr/bin/env python
#-*- coding:utf-8 -*-


#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#   Gimp-Python - allows the writing of GIMP plug-ins in Python.
#   Copyright (C) 1997  James Henstridge <james@daa.com.au>

# ------------
#| Change Log MrQ BoxText|
# ------------
# http://gimpchat.com/viewtopic.php?f=9&t=19519
# https://www.gimpscripts.net/2021/07/boxtext-ver10.html
# Rel 1: 29-07-2021 : MareroQ: Initial release.
# Rel 2: 01-09-2021 : add: Tab "Bend", Style for Gegl Long Shadow, Miter Limit & Shape Rendering for Stroke Line
# Rel 3: 17-06-2022 : add efect 3D by Tutorials PixLab: https://www.gimp-forum.net/Thread-3D-text-extrusion-effect-A-simple-way-without-plugins?pid=29804#pid29804
#                     preview is disabled by default after running the plug-in
# Rel 3.1: 20-06-2022 : add Colorize, Brightness and Contrast to tab "3 D"
# Rel 4:   21-06-2022 : rename tab '3 D' to '3 D Offset'
#					    add tab efect '3D Motion'
# Rel 4.1: 22-06-2022 : add option: 'Move blur center by X/Y px.\n(from center of image)' for tab '3D Motion'
# Rel 4.2: 25-06-2022 : bug fixes; add option: "An overlay from a file (scaled to image size)" to tab 'Fill'
# Rel 4.3: 26-06-2022 : add option "Change  Case" for tab 'TEXT':
#		  			    "lowercase" - lowercase text
#		  			    "UPPERCASE" - uppercase text
#		  			    "Capitalize" - first capital letter
#		  			    "Title" - the first letter of each word is capitalized
#		  			    "Swapcase" - Changes lowercase to uppercase, uppercase to lowercase
# Rel 4.4: 27-06-2022 : fixed bugs for "Bend" and "Long Shadow" when using '3 D Offset' or '3D Motion'
# Rel 5.0: 01-07-2022 : add option 'Zoom in on first letter' for tab 'TEXT'
#						add option Emboss type Bumpmap and opacity for tab 'Groov'
# Rel 5.1: 02-07-2022   reorganization of the "Fill" tab - add Effect: "Avarage Color","Canvas","Cubism","Edge","Edge-Neon", "Edge-Sobel", "Engrave","Gradient Map","Mosaic", "Pixelize","Plasma","Value-invert","Video Degradation"				
# Rel 5.2: 03-07-2022   add Vertical TEXT to tab "Bend"
# Rel 5.3: 05-07-2022   reorganization of the "Groove" tab - add BumpMap
#						added 2 new style envelope: '/\/\' and '\/\/' (now 324 possible combinations)
# Rel 6.0: 22-07-2022   reorganization of the "Stroke Path" tab - add ofn-gradient-along-path.py by Ofnuts https://sourceforge.net/projects/gimp-path-tools/files/scripts/
# Rel 6.1: 27-07-2022	add Waves: "Sine","Ellipse","Circle","Square","Triangle" by Ofnuts path-waves-0.2.py (must be installed independently)
# Rel 6.2: 13-08-2022	reorganization of the "Stickerify" tab (based on an idea https://www.gimpscripts.net/2022/08/p-creates-coloring-bevel.html)
# Rel 6.3: 15-08-2022	increase the number of colors to 7 in the "Stickerify" tab and set the default to rainbow colors
#						reorganization of the ' 3 D Blur' &' 3 D Offset' tab to one tab " 3 D"
# 						remove Colorize, Brightness and Contrast from tab "3 D"
# Rel 6.4: 16-08-2022   add Effect: Crystal,Denim,Rainbowify (tab: Fill)
#                       add a choice of what to use the 3D effect for: 'Fill Layer','Stroke Active Brush (if used)','Gradient along path (if used)'
#####################################################################################################################################################
############################################# START fu_preview 0.1.1 by Claude Lion $ J.F. Garcia ###################################################
#####################################################################################################################################################

import string as _string
import math
import gimp
import gimpcolor
import colorsys
import re
from gimpenums import *
pdb = gimp.pdb
import os, time, sys
import gettext
t = gettext.translation("gimp20-python", gimp.locale_directory, fallback=True)	#MrQ for translate in dir "locle"
_ = t.ugettext																	#MrQ for translate in dir "locle"
import random
class error(RuntimeError): pass
class CancelError(RuntimeError): pass

PF_INT8        = PDB_INT8
PF_INT16       = PDB_INT16
PF_INT32       = PDB_INT32
PF_INT         = PF_INT32
PF_FLOAT       = PDB_FLOAT
PF_STRING      = PDB_STRING
PF_VALUE       = PF_STRING
PF_COLOR       = PDB_COLOR
PF_COLOUR      = PF_COLOR
PF_ITEM        = PDB_ITEM
PF_DISPLAY     = PDB_DISPLAY
PF_IMAGE       = PDB_IMAGE
PF_LAYER       = PDB_LAYER
PF_CHANNEL     = PDB_CHANNEL
PF_DRAWABLE    = PDB_DRAWABLE
PF_VECTORS     = PDB_VECTORS

PF_TOGGLE      = 1000
PF_BOOL        = PF_TOGGLE
PF_SLIDER      = 1001
PF_SPINNER     = 1002
PF_ADJUSTMENT  = PF_SPINNER

PF_FONT        = 1003
PF_FILE        = 1004
PF_BRUSH       = 1005
PF_PATTERN     = 1006
PF_GRADIENT    = 1007
PF_RADIO       = 1008
PF_TEXT        = 1009
PF_PALETTE     = 1010
PF_FILENAME    = 1011
PF_DIRNAME     = 1012
PF_OPTION      = 1013
PF_TAB         = 1014
PF_DRAW        = 1015
PF_TITLE       = 1016

_type_mapping = {
    PF_INT8        : PDB_INT8,
    PF_INT16       : PDB_INT16,
    PF_INT32       : PDB_INT32,
    PF_FLOAT       : PDB_FLOAT,
    PF_STRING      : PDB_STRING,
    PF_COLOR       : PDB_COLOR,
    PF_ITEM        : PDB_ITEM,
    PF_DISPLAY     : PDB_DISPLAY,
    PF_IMAGE       : PDB_IMAGE,
    PF_LAYER       : PDB_LAYER,
    PF_CHANNEL     : PDB_CHANNEL,
    PF_DRAWABLE    : PDB_DRAWABLE,
    PF_VECTORS     : PDB_VECTORS,

    PF_TOGGLE      : PDB_INT32,
    PF_SLIDER      : PDB_FLOAT,
    PF_SPINNER     : PDB_INT32,

    PF_FONT        : PDB_STRING,
    PF_FILE        : PDB_STRING,
    PF_BRUSH       : PDB_STRING,
    PF_PATTERN     : PDB_STRING,
    PF_GRADIENT    : PDB_STRING,
    PF_RADIO       : PDB_STRING,
    PF_TEXT        : PDB_STRING,
    PF_PALETTE     : PDB_STRING,
    PF_FILENAME    : PDB_STRING,
    PF_DIRNAME     : PDB_STRING,
    PF_OPTION      : PDB_INT32,
    PF_TAB         : PDB_STRING,
    PF_DRAW        : PDB_INT32,
    PF_TITLE       : PDB_STRING,
}

_obj_mapping = {
    PF_INT8        : int,
    PF_INT16       : int,
    PF_INT32       : int,
    PF_FLOAT       : float,
    PF_STRING      : str,
    PF_COLOR       : gimpcolor.RGB,
    PF_ITEM        : int,
    PF_DISPLAY     : gimp.Display,
    PF_IMAGE       : gimp.Image,
    PF_LAYER       : gimp.Layer,
    PF_CHANNEL     : gimp.Channel,
    PF_DRAWABLE    : gimp.Drawable,
    PF_VECTORS     : gimp.Vectors,

    PF_TOGGLE      : bool,
    PF_SLIDER      : float,
    PF_SPINNER     : int,

    PF_FONT        : str,
    PF_FILE        : str,
    PF_BRUSH       : str,
    PF_PATTERN     : str,
    PF_GRADIENT    : str,
    PF_RADIO       : str,
    PF_TEXT        : str,
    PF_PALETTE     : str,
    PF_FILENAME    : str,
    PF_DIRNAME     : str,
    PF_OPTION      : int,
    PF_TAB         : str,
    PF_DRAW        : int,
    PF_TITLE       : str,
}

_registered_plugins_ = {}


def register(proc_name, blurb, help, author, copyright, date, label,
             imagetypes, params, results, function,
             menu=None, domain=None, on_query=None, on_run=None):
    """This is called to register a new plug-in."""

    # First perform some sanity checks on the data
    def letterCheck(str):
        allowed = _string.letters + _string.digits + "_" + "-"
        for ch in str:
            if not ch in allowed:
                return 0
        else:
            return 1

    if not letterCheck(proc_name):
        raise error, "procedure name contains illegal characters"

    for ent in params:
        if len(ent) < 4:
            raise error, ("parameter definition must contain at least 4 "
                          "elements (%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != int:
            raise error, "parameter types must be integers"

        if not letterCheck(ent[1]):
            raise error, "parameter name contains illegal characters"

    for ent in results:
        if len(ent) < 3:
            raise error, ("result definition must contain at least 3 elements "
                          "(%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != type(42):
            raise error, "result types must be integers"

        if not letterCheck(ent[1]):
            raise error, "result name contains illegal characters"

    plugin_type = PLUGIN

    if (not proc_name.startswith("python-") and
        not proc_name.startswith("python_") and
        not proc_name.startswith("extension-") and
        not proc_name.startswith("extension_") and
        not proc_name.startswith("plug-in-") and
        not proc_name.startswith("plug_in_") and
        not proc_name.startswith("file-") and
        not proc_name.startswith("file_")):
           proc_name = "python-fu-" + proc_name

    # if menu is not given, derive it from label
    need_compat_params = False
    if menu is None and label:
        fields = label.split("/")
        if fields:
            label = fields.pop()
            menu = "/".join(fields)
            need_compat_params = True

            import warnings
            message = ("%s: passing the full menu path for the menu label is "
                       "deprecated, use the 'menu' parameter instead"
                       % (proc_name))
            warnings.warn(message, DeprecationWarning, 3)

        if need_compat_params and plugin_type == PLUGIN:
            file_params = [(PDB_STRING, "filename", "The name of the file", ""),
                           (PDB_STRING, "raw-filename", "The name of the file", "")]

            if menu is None:
                pass
            elif menu.startswith("<Load>"):
                params[0:0] = file_params
            elif menu.startswith("<Image>") or menu.startswith("<Save>"):
                params.insert(0, (PDB_IMAGE, "image", "Input image", None))
                params.insert(1, (PDB_DRAWABLE, "drawable", "Input drawable", None))
                if menu.startswith("<Save>"):
                    params[2:2] = file_params

    _registered_plugins_[proc_name] = (blurb, help, author, copyright,
                                       date, label, imagetypes,
                                       plugin_type, params, results,
                                       function, menu, domain,
                                       on_query, on_run)

def _query():
    for plugin in _registered_plugins_.keys():
        (blurb, help, author, copyright, date,
         label, imagetypes, plugin_type,
         params, results, function, menu, domain,
         on_query, on_run) = _registered_plugins_[plugin]

        def make_params(params):
            return [(_type_mapping[x[0]],
                     x[1],
                     _string.replace(x[2], "_", "")) for x in params]

        params = make_params(params)
        # add the run mode argument ...
        params.insert(0, (PDB_INT32, "run-mode",
                                     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"))

        results = make_params(results)

        if domain:
            try:
                (domain, locale_dir) = domain
                gimp.domain_register(domain, locale_dir)
            except ValueError:
                gimp.domain_register(domain)

        gimp.install_procedure(plugin, blurb, help, author, copyright,
                               date, label, imagetypes, plugin_type,
                               params, results)

        if menu:
            gimp.menu_register(plugin, menu)
        if on_query:
            on_query()

def getXtra(xtra, valName, defValue):
    p = defValue
    if xtra.get(valName):
        p = xtra.get(valName)
    return p

def _get_defaults(proc_name):
    import gimpshelf

    (blurb, help, author, copyright, date,
     label, imagetypes, plugin_type,
     params, results, function, menu, domain,
     on_query, on_run) = _registered_plugins_[proc_name]

    key = "python-fu-save--" + proc_name

    if gimpshelf.shelf.has_key(key):
        return gimpshelf.shelf[key]
    else:
        # return the default values
        return [x[3] for x in params]

def _set_defaults(proc_name, defaults):
    import gimpshelf

    key = "python-fu-save--" + proc_name
    gimpshelf.shelf[key] = defaults

def _interact(proc_name, start_params):
    (blurb, help, author, copyright, date, label, imagetypes, plugin_type, params, results, function, menu, domain, on_query, on_run) = _registered_plugins_[proc_name]

    def run_script(run_params):
        params = start_params + tuple(run_params)
        _set_defaults(proc_name, params)
        return apply(function, params)

    params = params[len(start_params):]

    # short circuit for no parameters ...
    if len(params) == 0:
         return run_script([])

    import pygtk
    pygtk.require('2.0')

    import gimpui, gtk
#    import pango
    gimpui.gimp_ui_init () # add MrQ
    defaults = _get_defaults(proc_name)
    defaults = defaults[len(start_params):]

    class EntryValueError(Exception):
        pass

    def warning_dialog(parent, primary, secondary=None):
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE, primary)
        if secondary:
            dlg.format_secondary_text(secondary)
        dlg.run()
        dlg.destroy()

    def error_dialog(parent, proc_name):
        import sys, traceback

        exc_str = exc_only_str = _("Missing exception information")
        try:
            etype, value, tb = sys.exc_info()
            exc_str = "".join(traceback.format_exception(etype, value, tb))
            exc_only_str = "".join(traceback.format_exception_only(etype, value))
        finally:
            etype = value = tb = None

        title = _("An error occurred running %s") % proc_name
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, title)
        dlg.format_secondary_text(exc_only_str)

        alignment = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
        alignment.set_padding(0, 0, 12, 12)
        dlg.vbox.pack_start(alignment)
        alignment.show()

        expander = gtk.Expander(_("_More Information"));
        expander.set_use_underline(True)
        expander.set_spacing(6)
        alignment.add(expander)
        expander.show()

        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled.set_size_request(-1, 200)
        expander.add(scrolled)
        scrolled.show()

        label = gtk.Label(exc_str)
        label.set_alignment(0.0, 0.0)
        label.set_padding(6, 6)
        label.set_selectable(True)
        scrolled.add_with_viewport(label)
        label.show()

        def response(widget, id):
            widget.destroy()

        dlg.connect("response", response)
        dlg.set_resizable(True)
        dlg.show()

    # define a mapping of param types to edit objects ...
    class StringEntry(gtk.Entry):
        def __init__(self, default="", **kwargs): # modif C.L.
            gtk.Entry.__init__(self)
            self.set_text(str(default))
            self.set_activates_default(True)
            #~ self.connect('paste-clipboard', lambda _: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            self.get_buffer().connect('inserted-text', lambda _1, _2, _3, _4: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            self.get_buffer().connect('deleted-text', lambda _1, _2, _3: kwargs['dialg'].la_previ.invalidate()) # modif C.L

        def get_value(self):
            return self.get_text()


    class LabelTitle(gtk.Label):
        #def __init__ (self, default="", **kwargs): # modif C.L. ???
        def __init__(self, default="", align = (0.0, 0.0)):		
            gtk.Label.__init__(self)
            self.set_label(str(default))
            self.set_use_markup(True)
            if align != None:
                self.set_alignment(align[0],align[1])
        def get_value(self):
            return self.get_label()

    class TextEntry(gtk.ScrolledWindow):
        def __init__ (self, default="", **kwargs): # modif C.L.
            gtk.ScrolledWindow.__init__(self)
            self.set_shadow_type(gtk.SHADOW_IN)

            self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            self.set_size_request(100, -1)

            self.view = gtk.TextView()
            self.add(self.view)
            self.view.show()
            self.buffer = self.view.get_buffer()
            self.set_value(str(default))
            self.buffer.connect('changed', lambda _: kwargs['dialg'].la_previ.invalidate()) # modif C.L.			

        def set_value(self, text):
            self.buffer.set_text(text)

        def get_value(self):
            return self.buffer.get_text(self.buffer.get_start_iter(), self.buffer.get_end_iter())

    class IntEntry(StringEntry):
        def get_value(self):
            try:
                return int(self.get_text())
            except ValueError, e:
                raise EntryValueError, e.args

    class FloatEntry(StringEntry):
            def get_value(self):
                try:
                    return float(self.get_text())
                except ValueError, e:
                    raise EntryValueError, e.args

    def precision(step):
        # calculate a reasonable precision from a given step size
        if math.fabs(step) >= 1.0 or step == 0.0:
            digits = 0
        else:
            digits = abs(math.floor(math.log10(math.fabs(step))));
        if digits > 20:
            digits = 20
        return int(digits)

    class SliderEntry(gtk.HScale):
        # bounds is (upper, lower, step)
        def __init__(self, default=0, bounds=(0, 100, 5), **kwargs): # modif C.L.
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1],
                                      step, 10 * step, 0)
            # modif C.L.
            def react(adj):
                return self.get_toplevel().la_previ.invalidate()
            #~ self.adj.connect('value-changed', lambda adj: self.get_toplevel().la_previ.invalidate())
            self.adj.connect('value-changed', react)
            # end modif C.L.
            gtk.HScale.__init__(self, self.adj)
            self.set_digits(precision(step))

        def get_value(self):
            return self.adj.value

    class SpinnerEntry(gtk.SpinButton):
        # bounds is (upper, lower, step)
        def __init__(self, default=10, bounds=(0, 100, 5), **kwargs): # modif C.L.
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1],
                                      step, 10 * step, 0)
            self.adj.connect('value-changed', lambda adj: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            gtk.SpinButton.__init__(self, self.adj, step, precision(step))


    class ToggleEntry(gtk.ToggleButton):
        #~ def __init__(self, default=0):
        def __init__(self, default=0,labels=[_("DON'T USE"),_("YES")], **kwargs): # modif C.L.
            
            gtk.ToggleButton.__init__(self)
            self.labels = labels
            self.label = gtk.Label(labels[0])
            self.add(self.label)
            self.label.show()

            #~ self.connect("toggled", self.changed)
            # modif C.L.
            def react(tog):
                self.changed(tog)
                return kwargs['dialg'].la_previ.invalidate()
            self.connect("toggled", react)
            # end modif C.L.


            self.set_active(default)

        def changed(self, tog):
            if tog.get_active():
                self.label.set_text(self.labels[1])
            else:
                self.label.set_text(self.labels[0])

        def get_value(self):
            return self.get_active()

    class RadioEntry(gtk.VBox):
        def __init__(self, default=0, items=((_("YES"), 1), (_("No"), 0)), **kwargs): # modif C.L.
            gtk.VBox.__init__(self, homogeneous=False, spacing=2)

            button = None

            for (label, value) in items:
                button = gtk.RadioButton(button, label)
                self.pack_start(button)
                button.show()

                button.connect("toggled", self.changed, value)

                if value == default:
                    button.set_active(True)
                    self.active_value = value

        def changed(self, radio, value):
            if radio.get_active():
                self.active_value = value

        def get_value(self):
            return self.active_value

    class ComboEntry(gtk.ComboBox):
        def __init__(self, default=0, items=(), **kwargs): # modif C.L.
            store = gtk.ListStore(str)
            for item in items:
                store.append([item])

            gtk.ComboBox.__init__(self, model=store)

            cell = gtk.CellRendererText()
            self.pack_start(cell)
            self.set_attributes(cell, text=0)

            self.set_active(default)
            self.connect('changed', lambda adj: kwargs['dialg'].la_previ.invalidate()) # modif C.L.

        def get_value(self):
            return self.get_active()

    def FileSelector(default="", title=None, **kwargs): # modif C.L.
        # FIXME: should this be os.path.separator?  If not, perhaps explain why?
        if default and default.endswith("/"):
            if default == "/": default = ""
            return DirnameSelector(default)
        else:
            return FilenameSelector(default, title=title, save_mode=False)

    class FilenameSelector(gtk.HBox):
        #gimpfu.FileChooserButton
        def __init__(self, default, save_mode=True, title=None, **kwargs): # modif C.L.
            super(FilenameSelector, self).__init__()
            if not title:
                self.title = _("Python-Fu File Selection")
            else:
                self.title = title
            self.save_mode = save_mode
            box = self
            self.entry = gtk.Entry()
            image = gtk.Image()
            image.set_from_stock(gtk.STOCK_FILE, gtk.ICON_SIZE_BUTTON)
            self.button = gtk.Button()
            self.button.set_image(image)
            box.pack_start(self.entry)
            box.pack_start(self.button, expand=False)
            self.button.connect("clicked", self.pick_file)
            if default:
                self.entry.set_text(default)

        def show(self):
            super(FilenameSelector, self).show()
            self.button.show()
            self.entry.show()

        def pick_file(self, widget):
            entry = self.entry
            dialog = gtk.FileChooserDialog(
                         title=self.title,
                         action=(gtk.FILE_CHOOSER_ACTION_SAVE
                                     if self.save_mode else
                                 gtk.FILE_CHOOSER_ACTION_OPEN),
                         buttons=(gtk.STOCK_CANCEL,
                                gtk.RESPONSE_CANCEL,
                                gtk.STOCK_SAVE
                                     if self.save_mode else
                                gtk.STOCK_OPEN,
                                gtk.RESPONSE_OK)
                        )
            dialog.set_alternative_button_order ((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))
            dialog.show_all()
            response = dialog.run()
            if response == gtk.RESPONSE_OK:
                entry.set_text(dialog.get_filename())
            dialog.destroy()

        def get_value(self):
            return self.entry.get_text()


    class DirnameSelector(gtk.FileChooserButton):
        def __init__(self, default="", **kwargs): # modif C.L.
            gtk.FileChooserButton.__init__(self,
                                           _("Python-Fu Folder Selection"))
            self.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
            if default:
                self.set_filename(default)

        def get_value(self):
            return self.get_filename()

    # modif C.L.
    class ColorSelector(gimpui.ColorSelector):
        def __init__(self, default=gimpcolor.RGB(1.0, 0, 0), **kwargs):
            ret = gimpui.ColorSelector.__init__(self, default)
            self.connect('color-changed', lambda _: kwargs['dialg'].la_previ.invalidate())
            return ret
    class FontSelector(gimpui.FontSelector):
        def __init__(self, default="Sans", **kwargs):
            ret = gimpui.FontSelector.__init__(self, default)
            self.connect('font-set', lambda _1, _2, _3: kwargs['dialg'].la_previ.invalidate())
            return ret
    class PatternSelector(gimpui.PatternSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.PatternSelector.__init__(self, default)
            self.connect('pattern-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    class GradientSelector(gimpui.GradientSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.GradientSelector.__init__(self, default)
            self.connect('gradient-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    class PaletteSelector(gimpui.PaletteSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.PaletteSelector.__init__(self, default)
            self.connect('palette-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    # end modif C.L.
    class DrawCanvas(gtk.DrawingArea):
        def __init__(self,default, size=(100,100)):
            gtk.DrawingArea.__init__(self)
            self.set_size_request(size[0],size[1])
        def get_value(self):
            return 0

    _edit_mapping = {
            PF_INT8        : IntEntry,
            PF_INT16       : IntEntry,
            PF_INT32       : IntEntry,
            PF_FLOAT       : FloatEntry,
            PF_STRING      : StringEntry,
            PF_COLOR       : ColorSelector, # modif C.L.
            PF_ITEM        : IntEntry,  # should handle differently ...
            PF_IMAGE       : gimpui.ImageSelector,
            PF_LAYER       : gimpui.LayerSelector,
            PF_CHANNEL     : gimpui.ChannelSelector,
            PF_DRAWABLE    : gimpui.DrawableSelector,
            PF_VECTORS     : gimpui.VectorsSelector,

            PF_TOGGLE      : ToggleEntry,
            PF_SLIDER      : SliderEntry,
            PF_SPINNER     : SpinnerEntry,
            PF_RADIO       : RadioEntry,
            PF_OPTION      : ComboEntry,

            PF_FONT        : FontSelector,
            PF_FILE        : FileSelector,
            PF_FILENAME    : FilenameSelector,
            PF_DIRNAME     : DirnameSelector,
            PF_BRUSH       : gimpui.BrushSelector,
            PF_PATTERN     : PatternSelector,  # modif C.L.
            PF_GRADIENT    : GradientSelector, # modif C.L.
            PF_PALETTE     : PaletteSelector,  # modif C.L.
            PF_TEXT        : TextEntry,
            PF_TAB         : TextEntry,
            PF_DRAW        : DrawCanvas,
            PF_TITLE       : LabelTitle,
    }
    # JFGARCIA - modified: to pass args to the on_run
    if on_run:
        on_run(start_params+tuple(params))

    dialog = gimpui.Dialog(proc_name, "python-fu", None, 0, None, proc_name, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))

    dialog.set_alternative_button_order((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))

    dialog.set_transient()

    # modif C.L.
    hboxPrincipal = gtk.HBox(spacing=10)
	
    #dialog.vbox.pack_start(hboxPrincipal)			        # modif MrQ
    swin = gtk.ScrolledWindow()                             # New line MrQ
    swin.add_with_viewport(hboxPrincipal)                   # New Line MrQ
    dialog.vbox.pack_start(swin)                            # New Line MrQ
    swin.show()                                             # New Line MrQ	

    hboxPrincipal.show()
    
    dialog.la_previ = gimpui.DrawablePreview(start_params[1])
    dialog.la_previ.set_update(False)						# disable Preview
    dialog.la_previ.set_size_request(512,512)               # New Line MrQ	
    dialog.la_previ.la_img = start_params[0]
    dialog.la_previ.la_layer = start_params[1]
    hboxPrincipal.pack_start(dialog.la_previ)

    dialog.set_default_size(1070, 680)  					# New Line MrQ
    dialog.la_previ.show()
    # end modif C.L.

    vbox = gtk.VBox(False, 2)
    vbox.set_border_width(2)
    #vbox.set_size_request(-10, 0)                           # modif C.L. + MrQ
    #~ dialog.vbox.pack_start(vbox)
    hboxPrincipal.pack_start(vbox) # modif C.L.
    vbox.show()

    if blurb:
        if domain:
            try:
                (domain, locale_dir) = domain
                trans = gettext.translation(domain, locale_dir, fallback=True)
            except ValueError:
                trans = gettext.translation(domain, fallback=True)
            blurb = trans.ugettext(blurb)
        box = gimpui.HintBox(blurb)
        vbox.pack_start(box, expand=False)
        box.show()

    table = gtk.Table(1, 2, False)
    table.set_row_spacings(2)
    table.set_col_spacings(2)
    vbox.pack_start(table, expand=True)
    table.show()

    notebook = gtk.Notebook()
    tables=[]
    for i in range(len(params)):
        if params[i][0]==PF_TAB:
            vals = params[i][4]
            currPage = params[i][3]

            for n in vals:
                tbl = gtk.Table(2, 2, False)
                tbl.set_row_spacings(5)
                tbl.set_col_spacings(5)
                tbl.show()
                tables.append({'table':tbl,'rows':0, 'totCols':0})
                notebook.append_page(tbl, gtk.Label(n))
                notebook.set_current_page(currPage)

    if len(tables)>0:
        table.attach(notebook, 0,6,0,1)
        notebook.show()
    else:
        tables.append({'table':table,'rows':0})
    # modif C.L.
    def ifinvalidated(p_la_previ):
        
        # get dimension
        #===============
        x, y = p_la_previ.get_position()
        w, h = p_la_previ.get_size()
        
        # Effet
        #=========
        # has alpha
        lb_alpha = pdb.gimp_drawable_has_alpha(p_la_previ.la_layer)
        
        # img, layer temporaires
        tempImg = pdb.gimp_image_new(w, h, 0)
        #~ tempLayer = pdb.gimp_layer_new(tempImg, w, h, 0, 'temp', 100, 0)
        tempLayer = pdb.gimp_layer_new(tempImg, w, h, lb_alpha, 'temp', 100, 0)
        pdb.gimp_image_insert_layer(tempImg, tempLayer, None, 0)
        
        # copie orig dans tempo
        rgnPreviIn = p_la_previ.la_layer.get_pixel_rgn(x, y, w, h)
        rgnTemp = tempLayer.get_pixel_rgn(0, 0, w, h)
        rgnTemp[0:w, 0:h] = rgnPreviIn[x:x+w, y:y+h]
        
        # effect
        tempImg.undo_group_start()
        tempImg.disable_undo()
        
        params = []

        try:
            for wid in edit_wids:
                params.append(wid.get_value())
        except EntryValueError:
            warning_dialog(dialog, _("Invalid input for '%s'") % wid.desc)
        else:
            try:
                paramsTot = (tempImg, tempLayer) + tuple(params)
                _set_defaults(proc_name, start_params + tuple(params))
                function(*paramsTot)
                
                res = tempImg.active_layer
                
                if not lb_alpha: pdb.gimp_layer_flatten(res)
                
                tempImg.undo_group_end()
                
                rgnRes = res.get_pixel_rgn(0, 0, w, h)
                
                # mise dans previ
                #================
                p_la_previ.draw_buffer(rgnRes[0:w, 0:h], int(3+lb_alpha)*w)
                
                pdb.gimp_image_delete(tempImg)
                
                #pdb.gimp_display_new(tempImg)
            except CancelError:
                pass
            except Exception as e:
                dialog.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
                error_dialog(dialog, proc_name)
                raise
    # end modif C.L.
    def response(dlg, id):
        if id == gtk.RESPONSE_OK:
            dlg.set_response_sensitive(gtk.RESPONSE_OK, False)
            dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, False)
            params = []
            try:
                for wid in edit_wids:
                    params.append(wid.get_value())
            except EntryValueError:
                warning_dialog(dialog, _("Invalid input for '%s'") % wid.desc)
            else:
                try:
                    dialog.res = run_script(params)
                except Exception:
                    dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
                    error_dialog(dialog, proc_name)
                    raise
        dialog.la_previ.set_update(False)	# disable Preview
        gtk.main_quit()
    dialog.la_previ.connect('invalidated', ifinvalidated) # modif C.L.
    dialog.connect("response", response)
    iniX=0
    row=0
    edit_wids = []

    for i in range(len(params)):
        paramsi = params[i]
        pf_type = paramsi[0]
        name, desc = (paramsi[1], paramsi[2])
        def_val = defaults[i]

        tab, incr, rows, connect, cols = (0, 1, 1, None, 1)
        if len(params[i])>5:
            xtra = params[i][5]
            incr = int(getXtra(xtra, 'newLine', 1))
            if incr==0 : cols = 1
            cols = getXtra(xtra, 'cols'   , cols)
            iniX = getXtra(xtra, 'iniCol' , iniX)
            rows = getXtra(xtra, 'rows'   , 1)
            tab  = getXtra(xtra, 'tab'    , 0)
            connect = getXtra(xtra,'connect', None)
        else:
            iniX = 0
        tbl = tables[tab]['table']
        row = tables[tab]['rows']


        if pf_type==PF_TAB:
            wid = _edit_mapping[pf_type](None)
        else:
            if pf_type != PF_TITLE:
                label = gtk.Label(desc)
                label.set_use_underline(True)
                label.set_alignment(0.0, 0.5)
                tbl.attach(label, iniX + 1, iniX + 2, row, row + rows, xoptions = gtk.FILL)
                label.show()

            if pf_type in (PF_SPINNER, PF_SLIDER, PF_RADIO, PF_OPTION, PF_DRAW):
                wid = _edit_mapping[pf_type](def_val, params[i][4], dialg=dialog) # modif C.L.
            elif pf_type==PF_TOGGLE:
                if isinstance(paramsi[4], list):
                    wid = _edit_mapping[pf_type](def_val, paramsi[4],dialg=dialog)
                else:
                    wid = _edit_mapping[pf_type](def_val)				


            elif pf_type==PF_TITLE:
                wid = _edit_mapping[pf_type](desc)
            else:
                wid = _edit_mapping[pf_type](def_val, dialg=dialog) # modif C.L.
            if pf_type!=PF_TITLE:
                label.set_mnemonic_widget(wid)
                tbl.attach(wid, iniX + 2, iniX + 2 + cols, row, row + rows, yoptions=0)
                iniX = iniX + cols + 1
            else:
                tbl.attach(wid, iniX, iniX+cols, row, row + rows, xoptions=gtk.FILL)
            # jfGarcia: strip html tags from toolTip
            tTip = re.sub('<[^<]+?>', '', desc)
            if pf_type != PF_TEXT:
                wid.set_tooltip_text(tTip)
            else:
                # Attach tip to TextView, not to ScrolledWindow
                wid.view.set_tooltip_text(tTip)

            wid.set_name(name)
            if connect:
                try:
                    wid.connect(connect['signal'],connect['function'])
                except Exception, e:
                    pass
            wid.show()
            wid.desc = desc

        edit_wids.append(wid)

        tables[tab]['rows'] += incr
        if incr==1:
           iniX=0

    progress_vbox = gtk.VBox(False, 6)
    vbox.pack_end(progress_vbox, expand=False)
    progress_vbox.show()

    progress = gimpui.ProgressBar()
    progress_vbox.pack_start(progress)
    progress.show()
	
    dialog.show()

    gtk.main()

    if hasattr(dialog, "res"):
        res = dialog.res
        dialog.destroy()
        return res
    else:
        dialog.destroy()
        raise CancelError

def _run(proc_name, params):
    run_mode = params[0]
    func = _registered_plugins_[proc_name][10]

    if run_mode == RUN_NONINTERACTIVE:
        return apply(func, params[1:])

    script_params = _registered_plugins_[proc_name][8]

    min_args = 0
    if len(params) > 1:
        for i in range(1, len(params)):
            param_type = _obj_mapping[script_params[i - 1][0]]
            if not isinstance(params[i], param_type):
                break

        min_args = i

    if len(script_params) > min_args:
        start_params = params[:min_args + 1]

        if run_mode == RUN_WITH_LAST_VALS:
            default_params = _get_defaults(proc_name)
            params = start_params + default_params[min_args:]
        else:
            params = start_params
    else:
       run_mode = RUN_NONINTERACTIVE

    if run_mode == RUN_INTERACTIVE:
        try:
            res = _interact(proc_name, params[1:])
        except CancelError:
            return
    else:
        res = apply(func, params[1:])

    gimp.displays_flush()

    return res

def main():
    """This should be called after registering the plug-in."""
    gimp.main(None, None, _query, _run)

def fail(msg):
    """Display an error message and quit"""
    gimp.message(msg)
    raise error, msg

def N_(message):
    return message
	
	
#####################################################################################################################################################
############################################# END fu_preview 0.1.1 by Claude Lion $ J.F. Garcia #####################################################
#####################################################################################################################################################	
	

########################################### start ofn-path-merge-down #########################################################################
def mergeDown(image,sourcePath):
    image.undo_group_start()
    try:
        catch=False
        targetPath=None
        for p in image.vectors:
            if catch:
                targetPath=p 
                break
            if p.ID==sourcePath.ID:
                catch=True
        if targetPath:
            for s in sourcePath.strokes:
                points,closed=s.points
                pdb.gimp_vectors_stroke_new_from_points(targetPath,0, len(points),points,closed)
            pdb.gimp_image_remove_vectors(image,sourcePath)
    except Exception as e:
        print e.args[0]
        pdb.gimp_message(e.args[0])

    image.undo_group_end()
    return;
########################################### end ofn-path-merge-down ##########################################################################

#############################################################################################################################
def selection_to_gradient_border(image,layer,border_radius,gradient,):

	#save existing selection
	saved_selection_channel = pdb.gimp_selection_save(image)
	#creates a new layer to paint gradient with
	new_layer = pdb.gimp_layer_new(image,image.width,image.height,RGBA_IMAGE,'gradient layer',100,LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(image,new_layer,None,0)

	border_radius = int(border_radius)
	dist = []
	for i in range(border_radius-1,-1,-1):
		dist.append(float(i)/float(border_radius-1))
	num_color_samples,color_samples = pdb.gimp_gradient_get_custom_samples(gradient,border_radius,dist,FALSE)
	pdb.gimp_context_set_brush("2. Hardness 100");
	pdb.gimp_context_set_brush_hardness(1);
	pdb.gimp_context_set_brush_spacing(0.01);

	for i in range(border_radius-1,-1,-1):
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection_channel)

		ci = (border_radius - (i+1))*4
		color = (int(color_samples[ci+0]*255),int(color_samples[ci+1]*255),int(color_samples[ci+2]*255))
		pdb.gimp_context_set_foreground(color)

		stroke_width = i*2+1
		pdb.gimp_context_set_brush_size(stroke_width)
		pdb.gimp_drawable_edit_stroke_selection(new_layer)

	#Now that we have the mask selected we can remove the layer. and saved channel
	pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,saved_selection_channel)

	pdb.gimp_image_remove_channel(image,saved_selection_channel)
	#pdb.gimp_image_remove_layer(image,new_layer)
#############################################################################################################################

##############################################################################################################################################################################
########################################## Start Ofn-bend-path.py by Ofnuts ##################################################################################################
##############################################################################################################################################################################
import os, sys, math, cmath
from collections import namedtuple
import traceback

debug=False
def trace(s):
    if debug:
        print s

Curve=namedtuple('Curve',['startX','endX','startY','endY','coeffsX','coeffsY' ])

# Functions to determine the position of the reference lines
# All these functions return topY,botY
def referencesFromBestFit(image,path,topY,botY):
    # Due to Y pointing down the bottom is the biggest Y
    topY=min([min(stroke.points[0][1::2]) for stroke in path.strokes])
    botY=max([max(stroke.points[0][1::2]) for stroke in path.strokes])
    return topY,botY
    
def referencesFromExplicitValues(image,path,topY,botY):
    if topY>botY:
        return botY,topY
    return topY,botY

def referencesFromGuides(image,path,topY,botY):
    guide=0
    vPos=[]

    guide=pdb.gimp_image_find_next_guide(image,0)
    while guide!=0: 
        if pdb.gimp_image_get_guide_orientation(image, guide)==ORIENTATION_HORIZONTAL:
            vPos.append(pdb.gimp_image_get_guide_position(image, guide))
        guide=pdb.gimp_image_find_next_guide(image,guide)

    if len(vPos) != 2:
        raise Exception('There should be exactly two horizontal guides')
    return min(vPos),max(vPos)

referencesFunctions=[('Best fit',referencesFromBestFit),('Guides',referencesFromGuides),('Explicit Values',referencesFromExplicitValues)]

# The math

def cardano(d,c,b,a):

    J=cmath.exp(2j*cmath.pi/3)
    Jc=1/J
    z0=b/3/a
    a2,b2 = a*a,b*b
    p=-b2/3/a2 +c/a
    q=(b/27*(2*b2/a2-9*c/a)+d)/a
    D=-4*p*p*p-27*q*q
    r=cmath.sqrt(-D/27+0j)
    u=((-q-r)/2)**0.33333333333333333333333
    v=((-q+r)/2)**0.33333333333333333333333
    w=u*v
    w0=abs(w+p/3)
    w1=abs(w*J+p/3)
    w2=abs(w*Jc+p/3)
    if w0<w1:
        if w2<w0 : v*=Jc
    elif w2<w1 : v*=Jc
    else: v*=J
    return u+v-z0, u*J+v*Jc-z0, u*Jc+v*J-z0

def secondDegree(c,b,a):
    discrim=b*b-4*a*c
    A=(-b/(2*a))+0j # return complex number
    B=math.sqrt(abs(discrim))/(2*a)
    if discrim==0:
        return [A] # there should be only one
    elif discrim>0:
        return [A+B,A-B]
    else: # for completeness since these will be filtered out
        return [A+(B*1j),A-(B*1j)]

def l2r(s):

    if s[0]<s[-2]:
        return s
    else: # reverse the pairs
        return [c for p in reversed(zip(s[0::2],s[1::2])) for c in p]
    
def polynomialCoeffs(p):

    coeffs=[p[0],-3*p[0]+3*p[1],3*p[0]-6*p[1]+3*p[2],-p[0]+3*p[1]-3*p[2]+p[3]]
    return coeffs

def curveFromPoints(points):

    pX=points[0::2]
    pY=points[1::2]
    coeffsX=polynomialCoeffs(pX)
    trace('PointsX: %r, coeffsX: %r' % (pX,coeffsX))
    coeffsY=polynomialCoeffs(pY)
    return Curve(pX[0],pX[-1],pY[0],pY[-1],coeffsX,coeffsY)
    

class Spline(object):

    def __init__(self,stroke):
        self.name='***' # set later, once we know relative positions
        s,_=stroke.points
        s=l2r(s)
        self.curves=[]
        for i in range(2,len(s)-4,6):
            self.curves.append(curveFromPoints(s[i:i+8]))
            
    def dump(self):
        print '------------------ %s' % self.name
        for c in self.curves:
            print '%6.2f -> %6.2f' % (c.startX,c.endX)
            print c.coeffsX
            print c.coeffsY

    def leftY(self):
        return self.curves[0].startY
    
    def rightY(self):
        return self.curves[-1].endY
    
    def xOutsideRange(self,x):
        raise Exception('x=%.2f outside range for "%s" envelope' % (x,self.name))
                        
    def TforX(self,x,curve):

        coeffs=curve.coeffsX[:] # make copy since we alter
        coeffs[0]-=x # "Polynomial(t)=coord" is same as "Polynomial(t)-coord=0"
        d,c,b,a=coeffs
        if a!=0.:
            roots=cardano(d,c,b,a)
        elif b!=0:
            roots=secondDegree(d,c,b)
        elif c!=0:
            roots=[(-d/c)+0j]
        else:
            roots=[d+0j]
            
        #print "Cardano results:", roots
        realRoots=[root.real for root in sorted(roots,key=lambda r: r.real) if 0.<= root.real <= 1. and abs(root.imag) < 1e-8]
        if len(realRoots)==0:
            print 'Roots:',roots
            self.xOutsideRange(x)
        elif len(realRoots)>1:
            print 'Roots:',roots
            raise Exception('Invalid %s envelope shape for x=%.2f' % (self.name,x))
        else:
            return realRoots[0]
    
    def YforT(self,t,curve):

        p=curve.coeffsY
        termPower=1
        y=0
        for n in range(4):
            y+=termPower*p[n]
            termPower*=t
        return y
        
    def YforX(self,x):

        # Find adequate Bezier curve
        for c in self.curves:
            #trace('Checking Curve %.2f -> %.2f for %.2f on %s' % (c.startX,c.endX,x,self.name))
            if c.startX <= x <= c.endX:
                break
        else: # not found 
            self.xOutsideRange(x)

        # The two easy cases. 
        # Making them special cases may avoid round-off nastiness when solving the equations
        if x==c.startX:
            return c.startY
        if x==c.endX:
            return c.endY
        
        t=self.TforX(x,c)
        return self.YforT(t,c)

def coordBetween(c1,c2,r):
    return c1+(c2-c1)*r

def pointBetween(p1,p2,r):
    x1,y1=p1
    x2,y2=p2
    return coordBetween(x1,x2,r),coordBetween(y1,y2,r)

def addBendingPoints(pts,bendiness):

    # To handle the "closing" stroke from last to first without convoluted logic
    # extend each end with opposite end
    pts=pts[-2:]+pts+pts[:2]
    for c in range(0,len(pts)-1,3):
        sa,st,et,ea=pts[c:c+4]
        if sa==st and et==ea: # no tangents
            pts[c+1]=pointBetween(sa,ea,bendiness)
            pts[c+2]=pointBetween(ea,sa,bendiness)
    return pts[2:-2] # drop the ones we added, sight unseen

def bend(points,srcTopY,srcBotY,topSpline,botSpline,bendiness):
    pts=addBendingPoints(zip(points[0::2],points[1::2]),bendiness)
    bentPoints=[]
    # Move the Y corrdinate of each point
    for x,y in pts:
        yRelative=(y-srcTopY)/(srcBotY-srcTopY)
        bendTopY=topSpline.YforX(x)
        bendBotY=botSpline.YforX(x)
        bentY=bendTopY+(bendBotY-bendTopY)*yRelative
        #print 'x=%5.2f, y=%5.2f, yRelative=%5.3f, bendTopY=%5.2f, bendBotY=%5.2f, bentY=%5.2f' % (x,y,yRelative,bendTopY,bendBotY,bentY)
        bentPoints.extend([x,bentY])
    return bentPoints

def order(s1,s2):

    if s1.leftY()<=s2.leftY() and s1.rightY()<=s2.rightY():
        top,bottom=s1,s2 # Smaller Y at top 
    elif s1.leftY()>=s2.leftY() and s1.rightY()>=s2.rightY():
        top,bottom=s2,s1 
    else:
        raise Exception('Ambiguous position of the envelope strokes')
    top.name='top'
    bottom.name='bottom'
    if debug:
        top.dump()
        bottom.dump()
    return top,bottom

def splinesFromEnvelope(envelope):

    if len(envelope.strokes)!=2:
        raise Exception('Envelope path shoud contain exactly two strokes')
    s1=Spline(envelope.strokes[0])
    s2=Spline(envelope.strokes[1])
    return order(s1,s2)
    
def bendPath_text_logo(image,path,envelope,bendiness,references,topY,botY):
    trace('*'*60)
    # Figure out reference lines, and top/bottom envelopes
    srcTopY,srcBotY=referencesFunctions[references][1](image,path,topY,botY)
    bendiness=bendiness*.005 # 100% bendiness is half the distance     
    topSpline,botSpline=splinesFromEnvelope(envelope)
    
    # Bend each stroke
    result=gimp.Vectors(image,'Bent')
    for stroke in path.strokes:
        points,closed=stroke.points
        bentPoints=bend(points,srcTopY,srcBotY,topSpline,botSpline,bendiness)
        gimp.VectorsBezierStroke(result,bentPoints,closed)
    pdb.gimp_image_add_vectors(image,result,0)
    result.visible=True

##############################################################################################################################################################################
########################################## End Ofn-bend-path.py by Ofnuts ####################################################################################################
##############################################################################################################################################################################

#############################################################################################################################
############################################ Start code by Tin Tran #########################################################
#############################################################################################################################
	
def vector_to_line_stroke(image, vector, layer, color="#000000", width=1, capstyle="butt", joinstyle="miter", miterlimit=10, shaperendering="auto", remove="1"):
	import re, tempfile
	import os, string, sys
	newelements = {
			'stroke': color,
			'stroke-width': width,
			'stroke-linecap': capstyle,
			'stroke-linejoin': joinstyle,
			'stroke-miterlimit': miterlimit,
			'shape-rendering': shaperendering,
			}
	svg = pdb.gimp_vectors_export_to_string(image, vector)
	#fix width and height to be resolution (px/inch)-independent
	svg = re.sub(r'(<svg\s[^>]*\swidth\s*=\s*)\S*"', r'\1"%dpx"' % image.width, svg, flags=re.DOTALL)
	svg = re.sub(r'(<svg\s[^>]*\sheight\s*=\s*)\S*"', r'\1"%dpx"' % image.height, svg, flags=re.DOTALL)
	svg = re.sub(r'(<path\s[^>]*)\sstroke\s*=\s*"black"', r'\1', svg, flags=re.DOTALL)
	svg = re.sub(r'(<path\s[^>]*)\sstroke-width\s*=\s*"1"', r'\1', svg, flags=re.DOTALL)
	svg = re.sub(r'(<path\s)', r'\1' + ''.join([r'%s="%s" ' % i for i in newelements.items()]), svg, flags=re.DOTALL)
	tmpfile = tempfile.TemporaryFile(suffix=".svg")
	
	#gets just the file name to use, added by Tin Tran, tempFile.TemporaryFile above gave write permission error
	filename = os.path.basename(tmpfile.name)
	svgfile = gimp.directory +  os.sep + 'tmp' + os.sep + filename # MrQ
	#svgfile =  "~\\.gimp-2.8\\tmp\\" + filename
	svgfile = os.path.expanduser(svgfile)
	tmpfile = open(svgfile,"w")
	#end of addition by Tin Tran
	
	tmpfile.write(svg)
	tmpfile.flush();
	newlayer = pdb.gimp_file_load_layer(image, tmpfile.name)
	tmpfile.close()
	image.add_layer(newlayer) #needs to be added to the image to be able to copy from
	copyname = pdb.gimp_edit_named_copy(newlayer, "stroke")
	image.remove_layer(newlayer)
	floating_sel = pdb.gimp_edit_named_paste(layer, copyname, True)
	pdb.gimp_floating_sel_anchor(floating_sel)
	
	#added by Tin Tran, remove the file when we're done
	if remove==1:
		os.remove(svgfile)
############################################ End code by Tin Tran #########################################################		
		
###########################################################################################################################
########################################### START ofn-gradient-along-path.py ##############################################
###########################################################################################################################

# GIMP plugin to draw a gradient radiallly from a path
# More or less a reboot from neon-path, taking advantage
# of a new API to stroke lines
# (c) Ofnuts 2018
#   History:
#   v0.0: 2018-05-16: Initial version
#   v0.1: 2018-05-17: Add "Reverse gradient" option

CAP_STYLES=['Current','Butt','Round','Square']
JOIN_STYLES=['Current','Miter','Round','Bevel']
PRECISIONS=[('Very fine',.5),('Fine',1.),('Rough',2.),('Coarse',5.)]

def computeWidths(width,precision):
    w=width
    widths=[]
    while w>0:
        widths.append(w)
        w-=precision
    return widths

def computeGradientSamples(widths,width):
    raw=[w/width for w in widths]
    delta=raw[-1]/(len(widths)-1)
    adjusted=[w-i*delta for i,w in enumerate(raw)]
    return adjusted
     
def sampleColors(gradientap,widths,width,reverse):
    pdb.gimp_context_set_gradient(gradientap)
    gradientSamples=computeGradientSamples(widths,width)
    count,flatColors=pdb.gimp_gradient_get_custom_samples(gradientap,len(gradientSamples),gradientSamples,not reverse)
    return [tuple(flatColors[i:i+4]) for i in range(0,count,4)]
    
def gradientAlongPathBox(image,path,gradientap,width,reverse,precision,capStyle,joinStyle,miterLimit):
    pdb.gimp_image_undo_group_start(image)
    pdb.gimp_context_push()
    precision=PRECISIONS[precision][1]
    
    try:
        pdb.gimp_context_set_stroke_method(STROKE_LINE)
        if capStyle:
            pdb.gimp_context_set_line_cap_style(capStyle-1)
        if joinStyle:
            pdb.gimp_context_set_line_join_style(joinStyle-1)
            if joinStyle==1: # ugly hard-coding of "Miter"
                pdb.gimp_context_set_line_miter_limit(float(miterLimit))

        widths=computeWidths(width,precision)
        passes=len(widths)
        if passes < 2:
            raise Exception('Line width to small for the requested precision')
            
        drawable=image.active_drawable
        gradientSamples=computeGradientSamples(widths,width)
        colors=sampleColors(gradientap,widths,width,not reverse)
        for color,width in zip(colors,widths):
            pdb.gimp_context_set_foreground(color)
            pdb.gimp_context_set_line_width(width)
            pdb.gimp_drawable_edit_stroke_item(drawable, path)

    except Exception as e:
        print e.args[0]
        pdb.gimp_message(e.args[0])

    pdb.gimp_context_pop()
    pdb.gimp_image_undo_group_end(image)

    return True	
	
################################################# END ofn-gradient-along-path.py ###################################
####################################################################################################################
	
####################################################################################################################
################# START gegl_command by Kevin Payne & Claude Lion [cli345] #########################################
####################################################################################################################

from sys import platform
from ctypes import *

def load_library (library_name):
	if platform == "linux" or platform == "linux2":
		library_name = library_name + '.so.0'
	elif platform == "win32":
		from ctypes.util import find_library
		library_name = find_library (library_name + "-0")
	else:
		raise BaseException ("TODO")
	return CDLL (library_name)

gimpC = load_library ('libgimp-2.0')


def gegl_command_preview(image, drawable, gegl_graph_string):
	
	class GeglBuffer(Structure):
		pass
	drawable_id = drawable.ID
	
	gegl = load_library ('libgegl-0.4')
	gegl.gegl_init (None, None)
	gimpC.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
	gimpC.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

	x = c_int (); y = c_int (); w = c_int (); h = c_int ()
	non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
	args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
	
	if non_empty:
		source = gimpC.gimp_drawable_get_buffer (drawable_id)
		target = gimpC.gimp_drawable_get_shadow_buffer (drawable_id)
		gegl.gegl_render_op (source, target, "gegl:gegl", *args)
		gegl.gegl_buffer_flush (target)
		gimpC.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
		gimpC.gimp_drawable_update (drawable_id, x, y, w, h)
		gimpC.gimp_displays_flush ()

####################################################################################################
################# END gegl_command #################################################################
####################################################################################################
import codecs

def rgb_to_hex(rgb_color):
	r = rgb_color[0]
	g = rgb_color[1]
	b = rgb_color[2]
	return '#%02x%02x%02x' % (r,g,b)


def mrq_box_text(img, layer,
	text, selectionCase, margin, font, font_size,color, direction, just, letter_spacing, line_spacing, enlarge, corect_size, offset_x, offset_y, create_path, 
	manyxy,manyx,manyy,ofset,wx,hy,scale,sx,sy,centered,angle_slope,flip,rotate,angle,mura,
	info_bend,run_bend,distance,style_top,style_bottom, info_vertical, vertical,info_wawe,select_waves,wave_length,amplitude_wave,
	fill_pattern,fill_pattern_s,text_pattern,text_pattern_opacity, grad,grad_s,gradient,border_radius, overlay, use_overlay,file_ov,ov_opacity, efect_inf, add_efect, efect_opacity,
	embos, run_g, std_dev_x, std_dev_y, type, gr_opacity, azimuth,elevation,depth,bump_info, run_bump,slant,azimuth_b,elevation_b,depth_b,waterlevel_b, ambient_b, compensate_b, invert_b,colorize_gr,color_hsl_gr,
	stick_ls,run_s, merge_s,color1,grow1,color2,grow2,color3,grow3,color4,grow4, color5,grow5,color6,grow6,color7,grow7,
	stroke, stroke_s, color_stroke,	stroke_svg,stroke_svg_s,strokecolor,strokewidth,capstyleSVG,joinstyleSVG,miterlimit,shaperenderingSVG,remove,
	gap_info,grad_along_path,gradientgap,widthgap,reversegap,precisiongap,capStylegap,joinStylegap,miterLimitgap,
	use_mer,motion_ls, motion, type_motion, motion_x, motion_y, length_m, motion_intensity,
	extrusion,run_d, thick_d, direct_d,
	blur_ls, blur, color_shadow, blurx, blury, movex, movey,gegl_ls, run_long_s, style, angle_ls, length_ls, color_ls,	
	help, ptab):
	
	pdb.gimp_image_set_active_layer(img,layer)
	
	for layer in img.layers:
		pdb.gimp_image_set_active_layer(img,layer)
		pdb.gimp_item_set_visible(layer, 0)

	w = pdb.gimp_image_width(img)
	h = pdb.gimp_image_height(img)	
	pdb.gimp_selection_none(img)
	wl = pdb.gimp_drawable_width(layer)
	hl = pdb.gimp_drawable_height(layer)	
	layerCopy = pdb.gimp_layer_copy(layer, False)
	pdb.gimp_image_add_layer(img, layerCopy, -1)

	gimp.set_foreground(color)
	
	text=text.lstrip(codecs.BOM_UTF8.decode("utf8", "strict"))
	if selectionCase==1:
		text=text.lower()
	elif selectionCase==2:
		text=text.upper()
	elif selectionCase==3:
		text=text.capitalize()		
	elif selectionCase==4:
		text=text.title()
	elif selectionCase==5:
		text=text.swapcase
	
	if enlarge==1:	
		text_layer = pdb.gimp_text_fontname(img, None, wx, hy, text[1:], margin, 1, font_size, PIXELS, font)	
	else:
		text_layer = pdb.gimp_text_fontname(img, None, wx, hy, text, margin, 1, font_size, PIXELS, font)
	pdb.gimp_displays_flush()
	
	# justification:
	pdb.gimp_text_layer_set_base_direction (text_layer, direction)
	pdb.gimp_text_layer_set_justification (text_layer, just)
	pdb.gimp_text_layer_set_letter_spacing(text_layer, letter_spacing)
	pdb.gimp_text_layer_set_line_spacing(text_layer, line_spacing)
	pdb.gimp_image_resize_to_layers(img)
	#pdb.gimp_layer_resize_to_image_size(text_layer)
	if enlarge==1:	

		text_layer_s = pdb.gimp_text_fontname(img, None, wx, hy, text[0], margin, 1, font_size+corect_size, PIXELS, font)
		width, height, ascent, descent = pdb.gimp_text_get_extents_fontname(text[0], font_size+corect_size, PIXELS, font)
		pdb.gimp_layer_set_offsets(text_layer, width-offset_x,height-ascent-offset_y)
		text_layer = pdb.gimp_image_merge_down(img, text_layer_s, 0)	
	if centered==1:
		x = (img.width - text_layer.width) / 2 + int(wx)
		y = (img.height - text_layer.height) / 2 + int(hy)
		text_layer.set_offsets(x, y)
	if centered==2:	# "Only Horizontal"	
		x = (img.width - text_layer.width) / 2 + int(wx)
		y = layer.offsets[1] + int(hy)
		text_layer.set_offsets(x, y)
	if centered==3:	# "Only Vertical"	
		x = layer.offsets[1] + int(wx)
		y = (img.height - text_layer.height) / 2 + int(hy)
		text_layer.set_offsets(x, y)		
	# vectors

	new_vectors = pdb.gimp_vectors_new(img, text_layer)
	if enlarge:
		pdb.gimp_image_select_item(img, 2, text_layer)		
		pdb.plug_in_sel2path(img, text_layer)
		active_vectors = pdb.gimp_image_get_active_vectors(img)
		text_layer.name=text
		pdb.gimp_item_set_name(active_vectors, text_layer.name)
		pdb.gimp_selection_none(img)
	else:
		active_vectors=pdb.gimp_vectors_new_from_text_layer(img, text_layer)
		pdb.gimp_image_insert_vectors(img, active_vectors, None, -1)
		
	active_vectors = pdb.gimp_image_get_active_vectors(img)
	active_vectors.visible = False
	
	if flip==1:
		pdb.gimp_item_transform_flip_simple(text_layer, 0, FALSE, w/2)
		pdb.gimp_item_transform_flip_simple(active_vectors, 0, TRUE,0)		
	if flip==2:
		pdb.gimp_item_transform_flip_simple(text_layer, 1, FALSE, h/2)
		pdb.gimp_item_transform_flip_simple(active_vectors, 1, TRUE,0)
		
	if rotate==1:		
		pdb.gimp_item_transform_rotate(text_layer, 45*0.01745329252 ,FALSE, w/2, h/2)
		pdb.gimp_item_transform_rotate(active_vectors, 45*0.01745329252, TRUE,0,0)		
	if rotate==2:
		pdb.gimp_item_transform_rotate_simple(text_layer, 0, FALSE, w/2, h/2)
		pdb.gimp_item_transform_rotate_simple(active_vectors, 0, TRUE,0,0)		
	if rotate==3:
		pdb.gimp_item_transform_rotate_simple(text_layer, 1, FALSE, w/2, h/2)
		pdb.gimp_item_transform_rotate_simple(active_vectors, 1, TRUE,0,0)		
	if rotate==4:
		pdb.gimp_item_transform_rotate_simple(text_layer, 2, FALSE, w/2, h/2)
		pdb.gimp_item_transform_rotate_simple(active_vectors, 2, TRUE,0,0)
	if rotate==5:		
		pdb.gimp_item_transform_rotate(text_layer, angle*0.01745329252 ,FALSE, w/2, h/2)
		pdb.gimp_item_transform_rotate(active_vectors, angle*0.01745329252, TRUE,0,0)

	iloop, loop = 0,0
	if manyx!=1 or manyy!=1:
		pdb.gimp_image_undo_group_start(img)
		for loop in range(int(manyy)):

			for i_loop in range(int(manyx)):

				temp_layer = pdb.gimp_layer_new_from_drawable(text_layer, img)
				img.add_layer(temp_layer, 0)
				pdb.gimp_item_set_visible(temp_layer, False)
				pdb.gimp_item_transform_translate(temp_layer, i_loop*(text_layer.width),0+(loop*(text_layer.height)))
				pdb.gimp_item_set_visible(temp_layer, True)
				active_vector = pdb.gimp_image_get_active_vectors(img)
				new_vector = pdb.gimp_vectors_copy(active_vector) # copy the user supplied vector
				pdb.gimp_image_insert_vectors(img, new_vector, None, 0)
				pdb.gimp_item_transform_translate(active_vector, i_loop*(temp_layer.width),(loop*(temp_layer.height)))
				mergeDown(img,active_vector)
			
		pdb.gimp_image_remove_vectors(img, new_vector)	
		text_layer = pdb.gimp_image_merge_visible_layers(img,1)
		pdb.gimp_image_raise_item_to_top(img, text_layer)		
		pdb.gimp_image_undo_group_end(img)
	
	black = (0,0,0)
	white = (255,255,255)
	
	if sx!=0 or sy!=0:

		pdb.gimp_layer_scale(text_layer, text_layer.width + sx, text_layer.height + sy, True)
		pdb.gimp_image_select_item(img,CHANNEL_OP_REPLACE,text_layer)
		pdb.plug_in_sel2path(img, text_layer, run_mode=RUN_NONINTERACTIVE)
		pdb.gimp_selection_none(img)
		pdb.gimp_image_remove_vectors(img, active_vectors)
	
	if angle_slope !=0:
			
		pdb.gimp_layer_resize_to_image_size(text_layer)
		wt = pdb.gimp_drawable_width(text_layer)
		ht = pdb.gimp_drawable_height(text_layer)
		corner = math.fabs(angle_slope)			
		rad =  corner*0.01745329252
		an = h*math.tan(rad)
	#python_character_layers_from_text(img, text_layer, just)
		if angle_slope > 0:			
			 pdb.gimp_item_transform_perspective(text_layer, an/2, 0,   wt + an/2, 0,   -an/2, ht,    wt-an/2, ht)
			 active_vector = pdb.gimp_image_get_active_vectors(img)
			 pdb.gimp_item_transform_perspective(active_vector, an/2, 0,   wt + an/2, 0,   -an/2, ht,    wt-an/2, ht)			 
		else:
			 pdb.gimp_item_transform_perspective(text_layer, -an/2, 0,   w - an/2, 0,   an/2, ht,    w+an/2, ht)
			 active_vector = pdb.gimp_image_get_active_vectors(img)			 
			 pdb.gimp_item_transform_perspective(active_vector, -an/2, 0,   w - an/2, 0,   an/2, ht,    w+an/2, ht)
	
	pdb.gimp_selection_none(img)

	if mura>1:
		
		rotate_angle=0

		for item in range(0,int(mura)):
			
			inc_angle=360/mura
			rotate_angle=rotate_angle + inc_angle
			pdb.gimp_image_get_active_layer(img)
			copy = pdb.gimp_layer_copy(text_layer, TRUE)
			pdb.gimp_image_insert_layer(img, copy, None, -1)
			pdb.gimp_item_transform_rotate(copy, rotate_angle*0.01745329252 ,FALSE, w/2, h/2)
			active_vector = pdb.gimp_image_get_active_vectors(img)
			new_vector = pdb.gimp_vectors_copy(active_vector) # copy the user supplied vector
			pdb.gimp_image_insert_vectors(img, new_vector, None, -1)
			pdb.gimp_item_transform_rotate(active_vector, rotate_angle*0.01745329252 ,FALSE, w/2, h/2)
			mergeDown(img,active_vector)
			
		pdb.gimp_image_resize_to_layers(img)		
		pdb.gimp_displays_flush()
		pdb.gimp_image_remove_vectors(img, new_vector)
		text_layer = pdb.gimp_image_merge_visible_layers(img,1)
		pdb.gimp_image_raise_item_to_top(img, text_layer)
		
	if create_path==0:			
		for v in img.vectors:
			v.visible = 1
	pdb.gimp_item_set_visible(layer, True)

	if run_bend==1:

		if vertical==1 and direction==5:	
			pdb.gimp_selection_none(img)
			pdb.gimp_image_resize_to_layers(img)	
			new_vectors = pdb.gimp_vectors_new(img, text_layer)
			pdb.gimp_image_select_item(img, 2, text_layer)		
			pdb.plug_in_sel2path(img, text_layer)
			pdb.gimp_selection_none(img)			
			active_vectors=pdb.gimp_image_get_active_vectors(img)
			pdb.gimp_item_set_name(active_vectors, "Vertical Path")			
			#pdb.gimp_image_insert_vectors(image, x, None, -1)
			pdb.gimp_item_transform_rotate_simple(active_vectors,0, True, 1,1)
			pdb.gimp_item_transform_rotate_simple(text_layer,0, True, 1,1)		
		
		pdb.gimp_image_resize_to_layers(img)
		layer_bend = gimp.Layer(img, "Background", img.width, img.height, RGBA_IMAGE, 100, 0)
		img.add_layer(layer_bend, 0)

		# create path envelope	
		layer_copy = pdb.gimp_layer_new_from_drawable(text_layer, img)
		img.add_layer(layer_copy, 0)
		pdb.gimp_layer_flatten(layer_copy)
		pdb.gimp_image_select_item(img, 2, layer_copy)
		
		selection = pdb.gimp_selection_bounds(img)

		x1 = selection[1];
		y1 = selection[2];
		x2 = selection[3];
		y2 = selection[4];
		pdb.gimp_image_remove_layer(img, layer_copy)
		WS = (x1+(x2 - x1)/2)
		HS = (y1+(y2 - y1)/2)
		
		W2 =(x2 - x1)/2
		H2 =(y2 - y1)/2
		
		W3 =(x2 - x1)/3
		H3 =(y2 - y1)/3
		
		W4 =(x2 - x1)/4
		H4 =(y2 - y1)/4
		
		r=distance
		new_vectors_e = pdb.gimp_vectors_new(img,"Envelope")
		pdb.gimp_image_insert_vectors(img, new_vectors_e, None, -1)
		
		pdb.gimp_item_set_visible(new_vectors_e, True)	

		if style_top==0:    # Bulging up                                           
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y1, x1,y1, x1,y1,  x1+W4,y1-r, WS,y1-r, x2-W4,y1-r,  x2,y1, x2,y1 ,x2,y1, ), 0)
		if style_top==1:	# Bulging down
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y1-r, x1,y1-r, x1,y1-r,   x1+W4,y1, WS,y1, x2-W4,y1,   x2,y1-r, x2,y1-r, x2,y1-r), 0)	
		if style_top==2:	# A straight line
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1-r, x1,y1-r, x1,y1-r,   x2,y1-r, x2,y1-r ,x2,y1-r, ), 0)	
		if style_top==3:	# Bent up (roof)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y1, x1,y1, x1,y1,  WS,y1-r, WS,y1-r, WS,y1-r,  x2,y1, x2,y1 ,x2,y1,), 0)
		if style_top==4:	# Bent down (wedge)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y1-r, x1,y1-r, x1,y1-r,   WS,y1, WS,y1, WS,y1,   x2,y1-r, x2,y1-r, x2,y1-r), 0)	
		if style_top==5:	# Slash /
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1, x1,y1, x1,y1,  x2,y1-r, x2,y1-r, x2,y1-r), 0)	
		if style_top==6:	# BackSlash \
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1-r, x1,y1-r, x1,y1-r,   x2,y1, x2,y1 ,x2,y1 ), 0)	
		if style_top==7:	# Wave (tilde)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y1, x1,y1, x1,y1,    x1+W4-r,y1-r,x1+W4,y1-r,x1+W4+r,y1-r,     WS,y1, WS,y1, WS,y1,    x2-W4-r,y1+r, x2-W4,y1+r, x2-W4+r,y1+r,     x2,y1, x2,y1 ,x2,y1, ), 0)	
		if style_top==8:	# Wave (reversed tilde)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y1, x1,y1, x1,y1,    x1+W4-r,y1+r,x1+W4,y1+r,x1+W4+r,y1+r,     WS,y1, WS,y1, WS,y1,    x2-W4-r,y1-r,x2-W4,y1-r,x2-W4+r,y1-r,     x2,y1, x2,y1 ,x2,y1, ), 0)		
		if style_top==9:	# Tortoise shell
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y1, x1,y1, x1,y1,  x1+W3,y1-r, x1+W3,y1-r, x1+W3,y1-r,   x2-W3,y1-r, x2-W3,y-r, x2-W3,y1-r,   x2,y1, x2,y1 ,x2,y1, ), 0)	
		if style_top==10:	# Tortoise shell reversed
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y1-r, x1,y1-r, x1,y1-r,  x1+W3,y1, x1+W3,y1, x1+W3,y1,    x2-W3,y1, x2-W3,y1, x2-W3,y1,   x2,y1-r, x2,y1-r, x2,y1-r, ), 0)
		if style_top==11:	# Arc Trumpet right (upper)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1, x1,y1, x1,y1,   x2,y1, x2,y1-r ,x2,y1-r, ), 0)
		if style_top==12:	# Arc Trumpet left (upper)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1-r, x1,y1-r, x1,y1,   x2,y1, x2,y1 ,x2,y1, ), 0)
		if style_top==13:	# Fish right (upper)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y1-2*r, x1,y1-2*r, x1,y1-2*r,   x1+W3,y1, x1+W3,y1, x1+W3,y1,   x2-W3-2*r,y1-r, x2-W3,y1-r, x2-W3+2*r,y1-r,  x2,y1, x2,y1 ,x2,y1), 0)
		if style_top==14:	# Fish left (upper)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y1, x1,y1 ,x1,y1,    x1+W3-2*r,y1-r, x1+W3,y1-r, x1+W3+2*r,y1-r,   x2-W3,y1, x2-W3,y1, x2-W3,y1,   x2,y1-2*r, x2,y1-2*r, x2,y1-2*r,), 0)
		if style_top==15:	# binoculars (upper)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y1, x1,y1, x1,y1,    x1+W4-r,y1-r, x1+W4,y1-r, x1+W4+r,y1-r,   x1+W2,y1, x1+W2,y1, x1+W2,y1,    x2-W4-r,y1-r, x2-W4,y1-r, x2-W4+r,y1-r,    x2,y1, x2,y1 ,x2,y1), 0)		
		if style_top==16:	# /\/\ double roof(upper)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y1, x1,y1, x1,y1,    x1+W4,y1-r, x1+W4,y1-r, x1+W4,y1-r,   x1+W2,y1, x1+W2,y1, x1+W2,y1,    x2-W4,y1-r, x2-W4,y1-r, x2-W4,y1-r,    x2,y1, x2,y1 ,x2,y1), 0)
		if style_top==17:	# \/\/ shark teeth(upper)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y1-r, x1,y1-r, x1,y1-r,    x1+W4,y1, x1+W4,y1, x1+W4,y1,   x1+W2,y1-r, x1+W2,y1-r, x1+W2,y1-r,    x2-W4,y1, x2-W4,y1, x2-W4,y1,    x2,y1-r, x2,y1-r ,x2,y1-r), 0)

			
		if style_bottom==0:	    # Bulging up
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y2+r, x1,y2+r, x1,y2+r,   WS-W4,y2, WS,y2, WS+W4,y2,   x2,y2+r, x2,y2+r, x2,y2+r), 0)		
		if style_bottom==1:	    # Bulging down
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y2, x1,y2, x1,y2,   WS-W4,y2+r, WS,y2+r, WS+W4,y2+r,   x2,y2, x2,y2, x2,y2), 0)	
		if style_bottom==2:     # A straight line
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2+r, x1,y2+r, x1,y2+r,   x2,y2+r, x2,y2+r ,x2,y2+r), 0)		
		if style_bottom==3:	    # Bent up (roof)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y2+r, x1,y2+r, x1,y2+r,   WS,y2, WS,y2, WS,y2,   x2,y2+r, x2,y2+r, x2,y2+r), 0)	
		if style_bottom==4:	    # Bent down (wedge)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y2, x1,y2, x1,y2,   WS,y2+r, WS,y2+r, WS,y2+r,   x2,y2, x2,y2, x2,y2), 0)	
		if style_bottom==5:	    # Slash /
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2+r, x1,y2+r, x1,y2+r,    x2,y2, x2,y2, x2,y2), 0)
		if style_bottom==6:	    # BackSlash \
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2, x1,y2, x1,y2,  x2,y2+r, x2,y2+r, x2,y2+r), 0)
		if style_bottom==7:	    # Wave (tilde)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y2, x1,y2, x1,y2,  x1+W4-r,y2-r,x1+W4,y2-r,x1+W4+r,y2-r,  WS,y2, WS,y2, WS,y2,  x2-W4-r,y2+r, x2-W4,y2+r, x2-W4+r,y2+r,     x2,y2, x2,y2 ,x2,y2, ), 0)
		if style_bottom==8:	    # Wave (reversed tilde)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y2, x1,y2, x1,y2,    x1+W4-r,y2+r, x1+W4,y2+r, x1+W4+r,y2+r,     WS,y2, WS,y2, WS,y2,    x2-W4-r,y2-r,x2-W4,y2-r,x2-W4+r,y2-r,     x2,y2, x2,y2 ,x2,y2, ), 0)		
		if style_bottom==9:	    # Tortoise shell
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y2, x1,y2, x1,y2,   x1+W3,y2-r, x1+W3,y2-r, x1+W3,y2-r,    x2-W3,y2-r, x2-W3,y2-r, x2-W3,y2-r,   x2,y2, x2,y2 ,x2,y2, ), 0)	
		if style_bottom==10:    # Tortoise shell reversed
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y2, x1,y2, x1,y2,    x1+W3,y2+r, x1+W3,y2+r,x1+W3,y2+r,   x2-W3,y2+r,x2-W3,y2+r,x2-W3,y2+r,  x2,y2, x2,y2 ,x2,y2, ), 0)
		if style_bottom==11:    # Arc Trumpet right (lower)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2, x1,y2, x1,y2,   x2,y2, x2,y2+r ,x2,y2+r, ), 0)	
		if style_bottom==12:    # Arc Trumpet left (lower)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2+r, x1,y2+r, x1,y2,  x2,y2, x2,y2 ,x2,y2,  ), 0)	
		if style_bottom==13:	# Fish right (lower)	                  (x1,y1-r, x1,y1-r, x1,y1-r,   x1+W3,y1, x1+W3,y1, x1+W3,y1,   x2-W3-2*r,y1-r, x2-W3,y1-r, x2-W3+2*r,y1-r,  x2,y1, x2,y1 ,x2,y1
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y2+2*r, x1,y2+2*r, x1,y2+2*r,   x1+W3,y2, x1+W3,y2, x1+W3,y2,    x2-W3-2*r,y2+r, 2*W3,y2+r, x2-W3+2*r,y2+r,   x2,y2, x2,y2,x2,y2, ), 0)		
		if style_bottom==14:	# Fish left (lower)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y2, x1,y2 ,x1,y2,    x1+W3-2*r,y2+r, x1+W3,y2+r, W3+2*r,y2+r,   x2-W3,y2, x2-W3,y2-r/2, x2-W3,y2-r/2,   x2,y2+2*r, x2,y2+2*r, x2,y2+2*r,), 0)
		if style_bottom==15:	# binoculars (lower)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y2, x1,y2, x1,y2,    x1+W4-r,y2+r, x1+W4,y2+r, x1+W4+r,y2+r,   x1+W2,y2, x1+W2,y2, x1+W2,y2,    x2-W4-r,y2+r, x2-W4,y2+r, x2-W4+r,y2+r,    x2,y2, x2,y2 ,x2,y2), 0)	
		if style_bottom==16:	# /\/\ double roof(lower)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y2+r, x1,y2+r, x1,y2+r,    x1+W4,y2, x1+W4,y2, x1+W4,y2,   x1+W2,y2+r, x1+W2,y2+r, x1+W2,y2+r,    x2-W4,y2, x2-W4,y2, x2-W4,y2,    x2,y2+r, x2,y2+r ,x2,y2+r), 0)		
		if style_bottom==17:	# \/\/ double roof(lower)
			pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y2, x1,y2, x1,y2,    x1+W4,y2+r, x1+W4,y2+r, x1+W4,y2+r,   x1+W2,y2, x1+W2,y2, x1+W2,y2,    x2-W4,y2+r, x2-W4,y2+r, x2-W4,y2+r,    x2,y2, x2,y2 ,x2,y2), 0)
		
		pdb.gimp_selection_none(img)
		active_vectors=img.vectors[-1]
		bendPath_text_logo(img ,active_vectors ,new_vectors_e , r, 0, 0, 0)

		active_vectors.visible = False
		new_vectors_e.visible= False
		pdb.gimp_selection_none(img)
		
		if vertical==1 and direction==5:

			pdb.gimp_item_transform_rotate_simple(active_vectors,2, True, 1,1)
			pdb.gimp_item_transform_rotate_simple(text_layer,2, True, 1,1)
			pdb.gimp_image_resize_to_layers(img)
			pdb.gimp_selection_none(img)
			
		pdb.gimp_layer_resize_to_image_size(text_layer)
		pdb.gimp_layer_resize_to_image_size(layer_bend)
		gimp.set_background(color)
		pdb.gimp_image_select_item(img, 2, active_vectors)
		pdb.gimp_edit_fill(layer_bend,0)
		pdb.gimp_selection_none(img)
		new_vectors = []
	pdb.gimp_layer_resize_to_image_size(pdb.gimp_image_get_layer_by_name(img, text_layer.name))
	
	
	if select_waves!=0 and run_bend==1:
		active_vectors=img.vectors[0]	

	if select_waves==1:	# Sine
		pdb.python_fu_path_periodic_sine_wave(img,active_vectors,wave_length,amplitude_wave,0,0,0)
	
	if select_waves==2:	# ellipse
		pdb.python_fu_path_periodic_ellipse_wave(img,active_vectors,wave_length,amplitude_wave,0,0,0)
		
	if select_waves==3:	# Circle
		pdb.python_fu_path_periodic_circle_wave(img,active_vectors,wave_length,amplitude_wave,0,0,0)
		
	if select_waves==4:	# Square
		pdb.python_fu_path_periodic_square_wave(img,active_vectors,wave_length,amplitude_wave,0,0,0)
		
	if select_waves==5:	# Triangle
		pdb.python_fu_path_periodic_triangle_wave(img,active_vectors,wave_length,amplitude_wave,0,0,0)		
		
		
	pdb.gimp_item_set_visible(text_layer, 0)
	patternLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Pattern", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_insert_layer(img, patternLayer, None, -1)	
	active_vectors = pdb.gimp_image_get_active_vectors(img)
	
	pdb.gimp_image_select_item(img,0,active_vectors)
	pdb.gimp_context_set_pattern(text_pattern)
	if fill_pattern_s==1:
		pdb.gimp_edit_fill(patternLayer,4)
	if fill_pattern_s==0:
		pdb.gimp_edit_fill(patternLayer,0)
	pdb.gimp_selection_none(img)
	text_layer=patternLayer		
	#if fill_pattern_s==0 and run_bend==1:

	if grad_s==1 and run_bend==0:
		pdb.gimp_layer_resize_to_image_size(text_layer)
		pdb.gimp_selection_none(img)
		pdb.gimp_image_select_item(img,0,text_layer)
		selection_to_gradient_border(img,text_layer,border_radius,gradient,)
		pdb.gimp_selection_none(img)
		
	if grad_s==1:
		pdb.gimp_layer_resize_to_image_size(text_layer)
		pdb.gimp_selection_none(img)
		pdb.gimp_image_select_item(img,0,active_vectors)
		selection_to_gradient_border(img,layer,border_radius,gradient,)
		pdb.gimp_selection_none(img)
	pdb.gimp_selection_none(img)

	if use_overlay==1 and file_ov!="":
		
		over=pdb.gimp_image_insert_layer(img, pdb.gimp_file_load_layer(img, file_ov), None, -1)
		over = img.active_layer
		pdb.gimp_item_set_name(over, "Overlay")
		pdb.gimp_layer_scale(over, img.width, img.height, 0)
		pdb.gimp_selection_none(img)
		pdb.gimp_layer_add_alpha(over)
		pdb.gimp_image_select_item(img,0,text_layer)
		pdb.gimp_selection_invert(img)	
		pdb.gimp_drawable_edit_clear(over)
		pdb.gimp_selection_none(img)
		pdb.gimp_layer_set_opacity(over,ov_opacity)
		text_layer=pdb.gimp_image_merge_down(img, over, 0)		

	if add_efect!=0:
		pdb.gimp_selection_none(img)	
		efect_layer = pdb.gimp_layer_new_from_drawable(text_layer, img)
		pdb.gimp_image_insert_layer(img, efect_layer, None, 0)
		pdb.gimp_item_set_name(efect_layer, "Efect layer")
		pdb.gimp_image_select_item(img, 2, efect_layer)		
	
	if add_efect==1:	# average

		r, _, _, _, _, _ = pdb.gimp_histogram(layer,HISTOGRAM_RED,0,255)
		g, _, _, _, _, _ = pdb.gimp_histogram(layer,HISTOGRAM_GREEN,0,255)
		b, _, _, _, _, _ = pdb.gimp_histogram(layer,HISTOGRAM_BLUE,0,255)
		pdb.gimp_context_set_foreground((int(r),int(g),int(b)))
		pdb.gimp_image_select_item(img,0,efect_layer)
		pdb.gimp_edit_fill(efect_layer,0)
		pdb.gimp_selection_none(img)
	if add_efect==2:	# Camouflage
		pdb.gimp_selection_grow(img, 1)
		pdb.plug_in_gmic_qt(img,efect_layer, 1, 0, 'fx_camouflage 9,12,100,30,46,33,75,90,65,179,189,117,255,246,158')
	if add_efect==3:	# canvas			
		gegl_command_preview(img, efect_layer, 'texturize-canvas')	
	if add_efect==4:	# crystal
		pdb.plug_in_gmic_qt(img,efect_layer, 1, 0, 'fx_crystal_background 10,25,0,100,1')			
	if add_efect==5:	# cubism			
		gegl_command_preview(img, efect_layer, 'cubism')
	if add_efect==6:	# Denim
		pdb.plug_in_gmic_qt(img,efect_layer, 1, 0,'Denim_samj 5,2,0,40,40,25,50,0,43,108,126,255')		
	if add_efect==7:	# edge			
		gegl_command_preview(img, efect_layer, 'edge')
	if add_efect==8:	# edge-neon			
		gegl_command_preview(img, efect_layer, 'edge-neon')		
	if add_efect==9:	# edge-sobel		
		gegl_command_preview(img, efect_layer, 'edge-sobel')	
	if add_efect==10:	# engrave rgb-clip	
		gegl_command_preview(img, efect_layer, 'engrave rgb-clip')
	if add_efect==11:	# gradmap		
		pdb.plug_in_gradmap(img, efect_layer)
	if add_efect==12:	# apply_mosaic
		gegl_command_preview(img, efect_layer, 'mosaic')
	if add_efect==13:	# pixelize
		gegl_command_preview(img, efect_layer, 'pixelize')		
	if add_efect==14:	# plasma
		gegl_command_preview(img, efect_layer, 'plasma')
	if add_efect==15:	# rainbow
		pdb.gimp_selection_grow(img, 1)
		pdb.plug_in_gmic_qt(img,efect_layer, 1, 0, 'fx_rep_rainbowify 0,0,100,0,50,50')
	if add_efect==16:	# value-invert	
		gegl_command_preview(img, efect_layer, 'value-invert')	
	if add_efect==17:	# video-degradation	
		gegl_command_preview(img, efect_layer, 'video-degradation pattern=wide-striped additive=0')
	
	if add_efect!=0:
		pdb.gimp_layer_resize_to_image_size(efect_layer)
		pdb.gimp_layer_set_opacity(efect_layer,efect_opacity)
		text_layer=pdb.gimp_image_merge_down(img, efect_layer, 0)
		pdb.gimp_selection_none(img)		

	if run_g==1:
	
		if type==0:
			embos_type= "bumpmap"
		if type==1:
			embos_type= "emboss"
			
		layerCopy_tl = pdb.gimp_layer_copy(text_layer, False)
		pdb.gimp_image_add_layer(img, layerCopy_tl, 0)
		pdb.gimp_item_set_name(layerCopy_tl, "Groov")
		gegl_command_preview(img, layerCopy_tl, "gaussian-blur std-dev-x=%d std-dev-y=%d" % (std_dev_x, std_dev_y))
		gimp.progress_init('Embos' + '...')	
		gegl_command_preview(img, layerCopy_tl, "emboss type=%s azimuth=%d elevation=%d depth=%d" % (embos_type,azimuth, elevation, depth))
		pdb.gimp_curves_spline(layerCopy_tl, 0, 6, [0,0,219,128,255,255])
		#pdb.gimp_layer_set_mode(layerCopy_tl, LAYER_MODE_GRAIN_MERGE)
		pdb.gimp_item_set_name(layerCopy_tl, "Groov")
		pdb.gimp_layer_set_opacity(pdb.gimp_image_get_layer_by_name(img, "Groov"),gr_opacity)
		
	if run_bump:
	
		pdb.gimp_image_set_active_layer(img,text_layer)
		pdb.gimp_layer_resize_to_image_size(text_layer)
		pdb.gimp_image_select_item(img,CHANNEL_OP_REPLACE,text_layer)
		pdb.gimp_selection_feather(img,slant )	
		layer_map = pdb.gimp_selection_save(img)
		pdb.gimp_selection_none(img)

		#pdb.plug_in_bump_map(image, drawable, bumpmap, azimuth, elevation, depth, xofs, yofs, waterlevel, ambient, compensate, invert, type)
		pdb.plug_in_bump_map(img,text_layer,layer_map,azimuth_b,elevation_b,depth_b,0, 0,waterlevel_b, ambient_b, compensate_b, invert_b,0)
	
	if colorize_gr:
	
		pdb.gimp_image_set_active_layer(img,text_layer)
		h,l,s = colorsys.rgb_to_hls(color_hsl_gr[0]/255.0,color_hsl_gr[1]/255.0,color_hsl_gr[2]/255.0)
		hue = h * 360.0
		saturation = s * 100.0
		lightness = l * 200.0 - 100.0
		pdb.gimp_drawable_colorize_hsl(text_layer, hue, saturation, lightness)
		
	if run_s==1:
		new_layer_group = pdb.gimp_layer_group_new(img)
		parent = pdb.gimp_item_get_parent(new_layer_group)
		position = pdb.gimp_image_get_item_position(img, text_layer)
		pdb.gimp_image_insert_layer(img,new_layer_group, None, -1) 
		new_layer_group.name = "Stickerify"	
		pdb.gimp_image_select_item(img, 2, text_layer)
		
		if grow1>0:
			layer1 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Color 1", 100, LAYER_MODE_NORMAL)
			pdb.gimp_context_set_foreground(color1)
			pdb.gimp_image_insert_layer(img, layer1, None, 0)
			pdb.gimp_drawable_fill(layer1, FILL_TRANSPARENT)
			pdb.gimp_selection_grow(img, grow1)
			pdb.plug_in_sel2path(img, layer1)
			vectors = pdb.gimp_image_get_active_vectors(img)
			pdb.gimp_vectors_to_selection(vectors, 2, TRUE, FALSE, 0, 0)
			pdb.gimp_drawable_edit_fill(layer1, FILL_FOREGROUND)
			pdb.gimp_image_reorder_item(img, layer1, new_layer_group, position)
			pdb.gimp_image_remove_vectors(img, vectors)
			#pdb.gimp_image_lower_item_to_bottom(img, layer1)
		if grow2>0:
			layer2 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Color 2", 100, LAYER_MODE_NORMAL)
			pdb.gimp_context_set_foreground(color2)
			pdb.gimp_image_insert_layer(img, layer2, None, 0)
			pdb.gimp_drawable_fill(layer2, FILL_TRANSPARENT)
			pdb.gimp_selection_grow(img, grow2)
			pdb.plug_in_sel2path(img, layer2)
			vectors = pdb.gimp_image_get_active_vectors(img)
			pdb.gimp_vectors_to_selection(vectors, 2, TRUE, FALSE, 0, 0)
			pdb.gimp_drawable_edit_fill(layer2, FILL_FOREGROUND)
			pdb.gimp_image_reorder_item(img, layer2, new_layer_group, position)				
			pdb.gimp_image_lower_item_to_bottom(img, layer2)
			pdb.gimp_image_remove_vectors(img, vectors)			
		if grow3>0:
			layer3 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Color 3", 100, LAYER_MODE_NORMAL)
			pdb.gimp_context_set_foreground(color3)
			pdb.gimp_image_insert_layer(img, layer3, None, 0)
			pdb.gimp_drawable_fill(layer3, FILL_TRANSPARENT)
			pdb.gimp_selection_grow(img, grow3)
			pdb.plug_in_sel2path(img, layer3)
			vectors = pdb.gimp_image_get_active_vectors(img)
			pdb.gimp_vectors_to_selection(vectors, 2, TRUE, FALSE, 0, 0)
			pdb.gimp_drawable_edit_fill(layer3, FILL_FOREGROUND)
			pdb.gimp_image_reorder_item(img, layer3, new_layer_group, position)				
			pdb.gimp_image_lower_item_to_bottom(img, layer3)
			pdb.gimp_image_remove_vectors(img, vectors)			

		if grow4>0:
			layer4 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Color 4", 100, LAYER_MODE_NORMAL)
			pdb.gimp_context_set_foreground(color4)
			pdb.gimp_image_insert_layer(img, layer4, None, 0)
			pdb.gimp_drawable_fill(layer4, FILL_TRANSPARENT)
			pdb.gimp_selection_grow(img, grow4)
			pdb.plug_in_sel2path(img, layer4)
			vectors = pdb.gimp_image_get_active_vectors(img)
			pdb.gimp_vectors_to_selection(vectors, 2, TRUE, FALSE, 0, 0)
			pdb.gimp_drawable_edit_fill(layer4, FILL_FOREGROUND)
			pdb.gimp_image_reorder_item(img, layer4, new_layer_group, position)				
			pdb.gimp_image_lower_item_to_bottom(img, layer4)
			pdb.gimp_image_remove_vectors(img, vectors)			
			
		if grow5>0:
			layer5 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Color 5", 100, LAYER_MODE_NORMAL)
			pdb.gimp_context_set_foreground(color5)
			pdb.gimp_image_insert_layer(img, layer5, None, 0)
			pdb.gimp_drawable_fill(layer5, FILL_TRANSPARENT)
			pdb.gimp_selection_grow(img, grow5)
			pdb.plug_in_sel2path(img, layer5)
			vectors = pdb.gimp_image_get_active_vectors(img)
			pdb.gimp_vectors_to_selection(vectors, 2, TRUE, FALSE, 0, 0)
			pdb.gimp_drawable_edit_fill(layer5, FILL_FOREGROUND)
			pdb.gimp_image_reorder_item(img, layer5, new_layer_group, position)				
			pdb.gimp_image_lower_item_to_bottom(img, layer5)
			pdb.gimp_image_remove_vectors(img, vectors)
			
		if grow6>0:
			layer6 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Color 6", 100, LAYER_MODE_NORMAL)
			pdb.gimp_context_set_foreground(color6)
			pdb.gimp_image_insert_layer(img, layer6, None, 0)
			pdb.gimp_drawable_fill(layer6, FILL_TRANSPARENT)
			pdb.gimp_selection_grow(img, grow6)
			pdb.plug_in_sel2path(img, layer6)
			vectors = pdb.gimp_image_get_active_vectors(img)
			pdb.gimp_vectors_to_selection(vectors, 2, TRUE, FALSE, 0, 0)
			pdb.gimp_drawable_edit_fill(layer6, FILL_FOREGROUND)
			pdb.gimp_image_reorder_item(img, layer6, new_layer_group, position)				
			pdb.gimp_image_lower_item_to_bottom(img, layer6)
			pdb.gimp_image_remove_vectors(img, vectors)
			
		if grow7>0:
			layer7 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Color 7", 100, LAYER_MODE_NORMAL)
			pdb.gimp_context_set_foreground(color7)
			pdb.gimp_image_insert_layer(img, layer7, None, 0)
			pdb.gimp_drawable_fill(layer7, FILL_TRANSPARENT)
			pdb.gimp_selection_grow(img, grow7)
			pdb.plug_in_sel2path(img, layer7)
			vectors = pdb.gimp_image_get_active_vectors(img)
			pdb.gimp_vectors_to_selection(vectors, 2, TRUE, FALSE, 0, 0)
			pdb.gimp_drawable_edit_fill(layer7, FILL_FOREGROUND)
			pdb.gimp_image_reorder_item(img, layer7, new_layer_group, position)				
			pdb.gimp_image_lower_item_to_bottom(img, layer7)			
			pdb.gimp_image_remove_vectors(img, vectors)
			
		if merge_s==1:
			pdb.gimp_image_reorder_item(img, text_layer, new_layer_group, position)	
		text_layer=pdb.gimp_image_merge_layer_group(img, new_layer_group)
		pdb.gimp_image_lower_item(img, pdb.gimp_image_get_layer_by_name(img, "Stickerify"))

	pdb.gimp_image_select_item(img,CHANNEL_OP_REPLACE,text_layer)
	active_layer_position = pdb.gimp_image_get_item_position(img,text_layer)
	if stroke_s==1:		
		active_vectors = pdb.gimp_image_get_active_vectors(img)
		pdb.gimp_selection_none(img)
		W = pdb.gimp_image_width(img)
		H = pdb.gimp_image_height(img)
		pdb.gimp_context_set_foreground(color_stroke)
		stroke_b = gimp.Layer(img, "Stroke brush", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
		img.add_layer(stroke_b, -1)
		pdb.gimp_edit_stroke_vectors(stroke_b ,active_vectors)
		if use_mer==1:
			text_layer=pdb.gimp_image_merge_down(img, stroke_b, 0)

	
	if grad_along_path==1:
		W = pdb.gimp_image_width(img)
		H = pdb.gimp_image_height(img)
		gap = gimp.Layer(img, "GAP", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
		img.add_layer(gap, -1)	
	
		gradientAlongPathBox(img,active_vectors,gradientgap,widthgap,reversegap,precisiongap,capStylegap,joinStylegap,miterLimitgap)
		if use_mer==2 and run_s==0:
			text_layer=pdb.gimp_image_merge_down(img, gap, 0)
		if use_mer==2 and run_s==1:			
			text_layer=gap
	
		#creates shadow layer

		
	pdb.gimp_selection_none(img)
	

	
	if motion==1:
		x = (img.width - text_layer.width) / 2
		y = (img.height - text_layer.height) / 2
		text_layer.set_offsets(x, y)
		motion_copy = pdb.gimp_layer_new_from_drawable(text_layer,img)
		pdb.gimp_image_insert_layer(img,motion_copy,None,0)
		pdb.gimp_layer_resize_to_image_size(motion_copy)

		if type_motion==0:
			pdb.plug_in_mblur_inward(img,motion_copy,2,length_m, 0,img.width/2+ motion_x,img.height/2+motion_y)
		if type_motion==1:
			pdb.plug_in_mblur(img,motion_copy,2,length_m, 0,img.width/2+motion_x,img.height/2+motion_y)
		
		for i in range(0,int(motion_intensity)):
			motion_copy = pdb.gimp_layer_new_from_drawable(motion_copy,img)
			pdb.gimp_image_insert_layer(img,motion_copy,None,0)
			motion_copy = pdb.gimp_image_merge_down(img,motion_copy,EXPAND_AS_NECESSARY)
		motion_copy.name = "3D Motion"	
		pdb.gimp_image_lower_item(img, pdb.gimp_image_get_layer_by_name(img, "3D Motion"))
		
	if run_d:
		group_new = pdb.gimp_layer_group_new(img)
		parent = pdb.gimp_item_get_parent(group_new)
		position = pdb.gimp_image_get_item_position(img, text_layer)
		pdb.gimp_image_insert_layer(img,group_new, None, 0) 
		group_new.name = "3D Extrusion"
		pdb.gimp_image_lower_item(img, group_new)	
		
		for item in range(0,int(thick_d)):
			
			pdb.gimp_image_get_active_layer(img)
			copy = pdb.gimp_layer_copy(text_layer, TRUE)
			pdb.gimp_image_insert_layer(img, copy, None, 1)
			a,b = pdb.gimp_drawable_offsets(copy)
			pdb.gimp_image_set_active_layer(img,copy)
			if direct_d==0: # 0
				y=b-item
				pdb.gimp_layer_set_offsets(copy, a, y)
			if direct_d==1: # 45
				x=a+item
				y=b-item
				pdb.gimp_layer_set_offsets(copy, x, y)
			if direct_d==2: # 90
				x=a+item
				pdb.gimp_layer_set_offsets(copy, x, b)
			if direct_d==3: # 135
				x=a+item
				y=b+item
				pdb.gimp_layer_set_offsets(copy, x, y)
			if direct_d==4: # 180
				y=b+item
				pdb.gimp_layer_set_offsets(copy, a, y)
			if direct_d==5: # 225
				x=a-item
				y=b+item
				pdb.gimp_layer_set_offsets(copy, x, y)
			if direct_d==6: # 275
				x=a-item
				y=b-item
				pdb.gimp_layer_set_offsets(copy, x, b)
			if direct_d==7: # 315
				x=a-item
				y=b-item
				pdb.gimp_layer_set_offsets(copy, x, y)				

			pdb.gimp_image_reorder_item(img, copy, group_new, position)
			pdb.gimp_image_lower_item_to_bottom(img, copy)	
			pdb.gimp_layer_resize_to_image_size(copy)

		pdb.gimp_image_merge_layer_group(img, group_new)
		#pdb.gimp_image_resize_to_layers(img)
	if blur==1:	
		shadow_layer = pdb.gimp_layer_new(img,img.width,img.height,RGBA_IMAGE,"Shadow",80,LAYER_MODE_NORMAL)
		pdb.gimp_image_insert_layer(img,shadow_layer,None,active_layer_position+2)
		pdb.gimp_context_set_foreground(color_shadow)
		pdb.gimp_image_select_item(img,CHANNEL_OP_REPLACE,text_layer)
		pdb.gimp_edit_fill(shadow_layer,FILL_FOREGROUND)
		pdb.gimp_selection_none(img)
		pdb.plug_in_gauss(img,shadow_layer,blurx,blury,1)
		pdb.gimp_layer_translate(shadow_layer,movex,movey)
		pdb.gimp_layer_resize_to_image_size(shadow_layer)
		pdb.gimp_selection_none(img)
		
	if run_long_s==1:

		xg=pdb.gimp_image_get_active_layer(img)
		copy = pdb.gimp_layer_copy(xg, TRUE)
		pdb.gimp_image_insert_layer(img, copy, None, -1)
		pdb.gimp_item_set_name(copy, "Long Shadow")
		pdb.gimp_layer_resize_to_image_size(copy)
		color_to_gegl=rgb_to_hex(color_ls)
		
		if style==0:
			style_ls="finite"
		if style==1:
			style_ls="infinite"
		if style==2:
			style_ls="fading"
		if style==3:
			style_ls="fading-fixed-length"			
			
		gegl_command_preview(img, copy, "long-shadow style=%s angle=%d length=%d color=%s" % (style_ls,angle_ls, length_ls,color_to_gegl))
	if stroke_svg_s==1:	
		if capstyleSVG==0:
			capstyle='Butt'
		if capstyleSVG==1:
			capstyle='Round'
		if capstyleSVG==2:
			capstyle='Square'

#miter | miter-clip | round | bevel | arcs 
		if joinstyleSVG==0:
			joinstyle='Miter'
		if joinstyleSVG==1:
			joinstyle='Miter-clip'			
		if joinstyleSVG==2:
			joinstyle='Round'
		if joinstyleSVG==3:
			joinstyle='Bevel'
		if joinstyleSVG==4:
			joinstyle='Arcs'			
#'Auto','OptimizeSpeed','CrispEdges','GeometricPrecision'			
		if shaperenderingSVG==0:
			shaperendering='Auto'
		if shaperenderingSVG==1:
			shaperendering='OptimizeSpeed'			
		if shaperenderingSVG==2:
			shaperendering='CrispEdges'
		if shaperenderingSVG==3:
			shaperendering='GeometricPrecision'			
			
		pdb.gimp_selection_none(img)
		W = pdb.gimp_image_width(img)
		H = pdb.gimp_image_height(img)
		stroke = gimp.Layer(img, "Stroke SVG", W, H, RGBA_IMAGE, 100, NORMAL_MODE)
		img.add_layer(stroke, -1)			
		vector_to_line_stroke(img,active_vectors,stroke,rgb_to_hex(strokecolor),strokewidth,capstyle,joinstyle,miterlimit,shaperendering,remove)
		pdb.gimp_image_raise_item_to_top(img, stroke)
	
	if create_path==1:			
		for v in img.vectors:
			v.visible = 0
	if run_bend==1:
		pdb.gimp_item_set_visible(layer_bend, 0)		
#######################################################################################
	# add for "Preview"	
	view = pdb.gimp_layer_new_from_visible(img, img, "Preview from Visible")
	pdb.gimp_image_insert_layer(img, view, None, 0)
	
#######################################################################################	

hlp = "Resizing method 'Preview':\n1. Disable preview.\n2. Drag the corner.\n3. Turn on the preview again.\n\nAlso turn off 'Preview' when You want to change some parameters.\n\nOnly the size of the initial (active) layer is displayed, so the preview\nmay show an image slightly different from the final result. \n\nYou can apply multiple effects in one session\nby turning it on (Click to Start). \nSome options may double as to the result and the result\nmay not be visible if you activate several effects.\n\nThe plugin uses codes created by:\n<b>Claude Lion, Tin Tran, Ofnuts, Kevin Payne, J.F. Garcia.\n3D text extrusion effect by tutorial PixLab.</b>\nThank You very much for sharing.\n\nI wish You good use of this plugin:\n\nMareroQ."

register(
	"MrQ_BoxText", 		# name
	"MrQ_BoxText. Combine multiple text editing options into one plugin.Ver.6.4.\nSelect parameters, then turn on 'Preview'",   	#
	"Combine multiple text editing options into one plugin.",   	# help
	"MareroQ", 			# author
	"GPL", 				# copyright
	"2022",   			# date
	"BoxText...",       # menu name
	"RGB*",
	[

		(PF_IMAGE, "image",       "Input image", None),
		(PF_DRAWABLE, "drawable", "Input drawable", None),
		
		(PF_TEXT,     "text", "Text:", "Gimp\nscripts\n.net",0,{'newLine':'0'}),
		(PF_OPTION,   "selectionCase", "  Change\n  Case?", 0,["None","lowercase", "UPPERCASE", "Capitalize","Title", "Swapcase"],{'newLine':'1'}),		
		(PF_SPINNER,  "margin", "Add border (px):", 0, (0, 1000, 1),{'newLine':'0'}),
		(PF_FONT,     "font", "  Font:", "Arial",0,{'newLine':'1'}),
		(PF_SPINNER,  "font_size", "Font size (px):", 100, (10, 5000, 1),{'newLine':'0'}),
		(PF_COLOR,    "color","  Font Color: ", (255,255,255),0,{'newLine':'1'}),		
		(PF_OPTION,   "direction", "Direction:", 0, ["From left to right",  "From right to left", "Vertical, right to left\n(mixed orientation)", "Vertical, right to left\n(upright orientation)", "Vertical, left to right\n(mixed orientation)", "Vertical, left to right\n (upright orientation)\n [* add a space at\n end of the line!]"],{'newLine':'0'}),
		(PF_OPTION,   "just", "  Justify:", 2, ["LEFT","RIGHT","CENTER","FILL"],{'newLine':'1'}),
		(PF_SPINNER,  "letter_spacing", "Letter\nSpacing:", 0, (-5000, 5000, 1, 5, 0, 0),{'newLine':'0'}),	
		(PF_SPINNER,  "line_spacing", "  Line\n  Spacing:", 0, ( -5000, 5000, 1, 5, 0, 0),{'newLine':'1'}),
		(PF_TOGGLE ,  "enlarge", "Zoom in on\nfirst letter?", 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':'0'}),			
		(PF_SPINNER,  "corect_size", "  zoom size:", 1, (-1000, 1000, 1),{'newLine':'1'}),		
		(PF_SPINNER,  "offset_x", "  offset X (px):", 1, (-1000, 1000, 1),{'newLine':'0'}),	
		(PF_SPINNER,  "offset_y", "  offset Y:", 1, (-1000, 1000, 1),{'newLine':'1'}),
		(PF_TOGGLE ,  "create_path", "Create Paths", 1 ,[ " Visible"," Invisible"],{'newLine':'1'}),			
		
		(PF_TITLE  ,  "manyxy"  , "<b>                *** Repeat Sign (number of tiles) ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':1}),	
		(PF_SPINNER,  "manyx", "	Horizontally (x)", 1, (1, 100, 1),{'newLine':'0','tab':1}),
		(PF_SPINNER,  "manyy", " Vertical (y)", 1, (1, 100, 1),{'newLine':'1','tab':1}),	
		(PF_TITLE  ,  "ofset"  , "<b>\n                		*** Offset (pixels) ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':1}),			
		(PF_SPINNER,  "wx", "	In x direction", 0, (-1000, 1000, 1),{'newLine':'0','tab':1}),	
		(PF_SPINNER,  "hy", " In y direction", 0, (-1000, 1000, 1),{'newLine':'1','tab':1}),
		(PF_TITLE  ,  "scale"  , "<b>\n                	*** Scale (pixels by image) ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':1}),			
		(PF_SPINNER,  "sx", "	In x direction", 0, (-1000, 1000, 1),{'newLine':'0','tab':1}),	
		(PF_SPINNER,  "sy", " In y direction", 0, (-1000, 1000, 1),{'newLine':'1','tab':1}),	
		(PF_OPTION,   "centered", "CENTERED?", 1, ["NOT","YES", "Only Horizontal {||}", "Only Vertical {=}"],{'newLine':1,'tab':1}),
		(PF_SPINNER,  "angle_slope", "SHEAR (tilt angle): ", 0, (-45, 45, 1),{'newLine':1,'tab':1}),		
		(PF_OPTION,   "flip", "MIRROR\n(auto-center):", 0, ["Not mirror","HORIZONTAL "u'\u21e8'"", "VERTICAL "u'\u21e9'""],{'newLine':1,'tab':1}),	
		(PF_OPTION,   "rotate", "ROTATE\n(auto-center):", 0, [ "Not rotate", "ROTATE 45°","ROTATE 90°", "ROTATE 180°", "ROTATE 270°", "CUSTOM ANGLE  "u'\u21bb'"", ],{'newLine':'0','tab':1}),		
		(PF_SPINNER,  "angle", "  custom angle: ", 0, (-360, 360, 1),{'newLine':1,'tab':1}),	
		(PF_SPINNER,  "mura", "ROTATE IN A CIRCLE:\n(how 'MuRa's Copies') ", 1, (1, 360, 1),{'newLine':1,'tab':1}),

		(PF_TITLE  , "info_bend"  , "<b>					*** Bend ***</b>\n       (if Error: change 'Font size' and/or 'Intensity' and/or 'Add Border')",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':2}),
		(PF_TOGGLE , "run_bend" , "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'iniCol':0,'newLine':1,'tab':2}),
		(PF_SPINNER, "distance", "Intensity = max.\nbending distance:", 50, (0, 1000, 1),{'newLine':1,'tab':2}),	
		(PF_OPTION,   "style_top", "Top style:", 0, [
		u'\u25E0'"    Bulging up",
		u'\u25E1'"    Bulging down",
		u'\uFE58'"    A straight line",		
		u'\u2227'"    Bent up (roof)",
		u'\u2228'"    Bent down (wedge)",		
		u'\uFF0F'"    Slash",		
		u'\uFF3C'"    BackSlash",
		u'\u223C'"    Wave (tilde)",		
		u'\u223D'"    Wave (reversed tilde)",
		u'\uFE39'"    Tortoise shell",
		u'\uFE3A'"    Tortoise shell reversed",	
		u'\u25DE'"     Arc Trumpet right (upper)",
		u'\u25DF'"     Arc Trumpet left (upper)",
		u'\u21A3'"    Fish right (upper)",
		u'\u21A2'"    Fish left (upper)",
		u'\u25E0\u25E0'"  Binoculars (upper)",
		"/\/\   Double roof (upper)",
		"\/\/   Shark teeth (upper)"
		],{'newLine':1,'tab':2}),
		(PF_OPTION,   "style_bottom", "Bottom style:", 1, [
		u'\u25E0'"    Bulging up",
		u'\u25E1'"    Bulging down",
		u'\uFE58'"    A straight line",
		u'\u2227'"    Bent up (roof)",
		u'\u2228'"    Bent down (wedge)",
		u'\uFF0F'"    Slash",		
		u'\uFF3C'"    BackSlash",
		u'\u223C'"    Wave (tilde)",		
		u'\u223D'"    Wave (reversed tilde)",
		u'\uFE39'"    Tortoise shell",
		u'\uFE3A'"    Tortoise shell reversed",
		u'\u25DD'"     Arc Trumpet right (lower)",
		u'\u25DC'"     Arc Trumpet left (lower)",
		u'\u21A3'"    Fish right (lower)",
		u'\u21A2'"    Fish left (lower)",
		u'\u25E1\u25E1'"  Binoculars (lower)",
		"/\/\   Double roof (lower)",
		"\/\/   Shark teeth (lower)"		
		],{'newLine':1,'tab':2}),
		(PF_TITLE  , "info_vertical"  , "<b>                 		*** Bend Vertical TEXT ***</b>\n         (only for Direction: Vertical, left to right 'upright orientation')",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':2}),		
		(PF_TOGGLE , "vertical" , "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':1,'tab':2}),
		(PF_TITLE  , "info_wawe"  , "<b>                 			    *** Waves ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':2}),		
		(PF_OPTION,  "select_waves", "Wave Type:", 0, ["Don't Use","Sine","Ellipse","Circle","Square","Triangle"],{'newLine':1,'tab':2}),
		(PF_SPINNER, "wavelength", "Wavelength (px):", 10, (0, 1000, 1),{'newLine':1,'tab':2}),
		(PF_SPINNER, "amplitudewave", "Amplitude (px):", 5, (0, 1000, 1),{'newLine':1,'tab':2}),		

		(PF_TITLE  , "fill_pattern"  , "<b>			*** Fill Pattern ***</b>",(0.0,0.0), None ,{'cols':10,'iniCol':1,'newLine':1,'tab':3}),
		(PF_TOGGLE , "fill_pattern_s" , "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':1,'tab':3}),				
		(PF_PATTERN, "text_pattern", "  pattern:", "Crack",0,{'newLine':'0','tab':3,'iniCol':0,}),
		(PF_SPINNER, "text_pattern_opacity", "    opacity: ", 100, (10, 100, 1),{'newLine':'1','tab':3}),		
		(PF_TITLE  , "grad"  , "<b>\n			*** Gradient Border ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':3}),	
		(PF_TOGGLE , "grad_s" , "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':1,'tab':3}),		
		(PF_GRADIENT,"gradient", "  gradient:", 0,0, {'newLine':'0','tab':3}),		
		(PF_SPINNER, "border_radius", "    border-radius: ", 2, (2, 100, 1),{'newLine':'1','tab':3}),
		(PF_TITLE  , "overlay"  , "<b>\n	*** An overlay from a file (scaled to image size) ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':3}),	
		(PF_TOGGLE , "use_overlay" , "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':1,'tab':3}),
		(PF_FILE,    "file_ov", "  file:", 0,0,{'newLine':'0','tab':3}),
		(PF_SPINNER, "ov_opacity", "    opacity: ", 100, (10, 100, 1),{'newLine':'1','tab':3}),
		(PF_TITLE  , "efect_inf"  , "<b>			*** Add Effect ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':3}),		
		(PF_OPTION,  "add_efect",     "Effect:", 0, ["Don't Use","Avarage Color\n(from started layer)","Camouflage","Canvas","Crystal","Cubism","Denim", "Edge","Edge-Neon", "Edge-Sobel", "Engrave","Gradient Map\n(recolor from active gradient)","Mosaic", "Pixelize","Plasma","Rainbowify","Value-invert","Video Degradation"],{'newLine':'0','tab':3}),		
		(PF_SPINNER, "efect_opacity", "    opacity: ", 100, (10, 100, 1),{'newLine':1,'tab':3}),		

		(PF_TITLE  , "embos"  , "<b>   			    *** Emboss ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':4}),
		(PF_TOGGLE , "run_g" , ""                  , 0,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':1,'tab':4}),			
		(PF_SPINNER, "std_dev_x", 'Blur Size X: ', 1.5,(0,100,0.1),{'cols':0,'newLine':'0','iniCol':0,'tab':4}),  # std-dev-y type: double default: 1.50 minimum: 0.00 maximum: 1500.00
		(PF_SPINNER, "std_dev_y", '   Blur Size Y : ', 1.5, (0,100,0.1),{'newLine':'1','tab':4,'cols':0}),
		(PF_OPTION,  "embos_type",  "Emboss type:", 0, ["Bumpmap","Emboss"],{'newLine':'0','tab':4}),
		(PF_SPINNER, "gr_opacity", "   Opacity: ", 100, (10, 100, 1),{'newLine':'1','tab':4}),
		(PF_SPINNER, "azimuth", 'Azimuth: ', 135, (0,360,1),{'newLine':'0','tab':4}),             # azimuth type: double default: 30.00 minimum: 0.00 maximum: 360.00 
		(PF_SPINNER, "elevation", '   Elevation: ', 45, (0,180,1),{'newLine':'1','tab':4}),  	  # elevation type: double default: 45.00 minimum: 0.00 maximum: 180.00 
		(PF_SPINNER, "depth", 'Depth: ', 6, (1,100,1),	{'newLine':'1','tab':4}),                 # depth type: int default: 20 minimum: 1 maximum: 100 
		(PF_TITLE  , "bump_info"  , "<b>\n   			  *** Bump Map ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':4}),
		(PF_TOGGLE , "run_bump" , ""                  , 0,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':1,'tab':4}),		
		(PF_SPINNER, "slant",      "Radius of feather: ", 15, (0, 100, 1),{'newLine':'0','tab':4}), 	
		(PF_SPINNER, "azimuth_b",    "   Azimuth: ", 45, (0, 360, 1),{'newLine':'1','tab':4}), 	
		(PF_SPINNER, "elevation_b",  "Elevation: ", 27, (0.5, 90, 0.1),{'newLine':'0','tab':4}), 	
		(PF_SPINNER, "depth_b",      "   Depth: ", 65, (1, 65, 1),{'newLine':'1','tab':4}), 
		(PF_SPINNER, "waterlevel_b", "Waterlevel: ", 0, (0.0, 1, 0.1),{'newLine':'0','tab':4}), 	
		(PF_SPINNER, "ambient",    "   Ambient\n   lighting\n   factor: ", 0, (0, 1, 0.01),{'newLine':'1','tab':4}), 
		(PF_TOGGLE,  "compensate", "Compensate? ",1, [" DON'T USE "," YES "],{'newLine':'0','tab':4}), 	
		(PF_TOGGLE,  "invert",     "   Bump Invert? ",0, [" DON'T USE "," YES "],{'newLine':'1','tab':4}), 		
		(PF_TOGGLE,  "colorize_gr",   "Colorize?: ",0,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':'0','tab':4}),		
		(PF_COLOR,   "color_hsl_gr",  " Color [HSL]: ", (130, 148, 213),0,{'newLine':'1','tab':4}), 	

		(PF_TITLE  , "stick_ls"  , "<b>				*** STICKERIFY  ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':5}),		
		(PF_TOGGLE , "run_s" , "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':'0','tab':5}),
		(PF_TOGGLE , "merge_s" , "   Merge with\n   text layer? " , 0 ,[" DON'T USE "," YES "],{'newLine':1,'tab':5}),		
		(PF_COLOR,   "color1", "  Color 1", (148, 0, 211),0,{'newLine':'0','tab':5}),		# violet - fiolet
		(PF_SPINNER, "grow1", "   Grow 1 (px):", 5, (0, 1000, 1),{'newLine':'1','tab':5}),		
		(PF_COLOR,   "color2", "  Color 2", (75, 0, 130),0,{'newLine':'0','tab':5}),		# indigo - granat
		(PF_SPINNER, "grow2", "   Grow 2 (px):", 5, (0, 1000, 1),{'newLine':'1','tab':5}),		
		(PF_COLOR,   "color3", "  Color 3", (0, 0, 255),0,{'newLine':'0','tab':5}),			# blue - niebieski
		(PF_SPINNER, "grow3", "   Grow 3 (px):", 5, (0, 1000, 1),{'newLine':'1','tab':5}),		
		(PF_COLOR,   "color4", "  Color 4", (0, 255, 0), 0,{'newLine':'0','tab':5}),		# green - zielony
		(PF_SPINNER, "grow4", "   Grow 4 (px):", 5, (0, 1000, 1),{'newLine':'1','tab':5}),
		(PF_COLOR,   "color5", "  Color 5", (255, 255, 0), 0,{'newLine':'0','tab':5}),		# yellow - żółty 
		(PF_SPINNER, "grow5", "   Grow 5 (px):", 5, (0, 1000, 1),{'newLine':'1','tab':5}),
		(PF_COLOR,   "color4", "  Color 6", (255, 127, 0), 0,{'newLine':'0','tab':5}),		# orange - pomarańczowy 
		(PF_SPINNER, "grow4", "   Grow 6 (px):", 5, (0, 1000, 1),{'newLine':'1','tab':5}),
		(PF_COLOR,   "color4", "  Color 7", (255, 0, 0), 0,{'newLine':'0','tab':5}),		# red - czerwony 
		(PF_SPINNER, "grow4", "   Grow 7 (px):", 5, (0, 1000, 1),{'newLine':'1','tab':5}),

		(PF_TITLE  , "stroke"  , "<b>		                   *** Stroke With active brush ****</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':6}),		
		(PF_TOGGLE , "stroke_s" ,"", 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':'0','tab':6}),		
		(PF_COLOR,   "color_stroke", "   Color:", (100,256,256),0,{'newLine':1,'tab':6}),
		(PF_TITLE  , "stroke_svg"  , "<b>		                           **** Stroke Line ****</b> \n                              (using SVG method - creates svg files in folder 'tmp'):",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':6}),		
		(PF_TOGGLE , "stroke_svg_s" ,"", 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':'0','tab':6}),		
		(PF_COLOR,   "strokecolor",  "   Color:",  (0,0,0),0,{'newLine':1,'tab':6}),	
		(PF_SPINNER, "strokewidth", "Line Width (px):", 3, (1, 200, 1),{'newLine':'0','tab':6}),
		(PF_OPTION,  "capstyleSVG", "   Cap Style", 0, ['Butt','Round','Square'],{'newLine':1,'tab':6}),	
		(PF_OPTION,  "joinstyleSVG", "Join Style", 2, ["Miter","Miter-clip","Round","Bevel","Arcs"],{'newLine':'0','tab':6}),		
		(PF_SPINNER, "miterlimit", "   Miter Limit:", 10, (0, 100, 1),{'newLine':1,'tab':6}),		
		(PF_OPTION,  "shaperenderingSVG", "Shape Rendering ", 0, ["Auto","Optimize\nSpeed","Crisp\nEdges","Geometric\nPrecision"],{'newLine':'0','tab':6}),		
		(PF_TOGGLE , "remove", "   Remove svg files?" , 1 ,["DON'T USE","YES"],{'newLine':1,'tab':6}),	
		(PF_TITLE  , "gap_info"  , "<b>		                   *** Gradient along path ***</b>:",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':6}),		
		(PF_TOGGLE , "grad_along_path" ,"", 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':'0','tab':6}),		
		(PF_GRADIENT,"gradientap", "   Gradient:","Golden", 0,{'newLine':1,'tab':6}),	
		(PF_SPINNER, 'widthgap', 'Width (pixels):', 20, (1, 2000, 1),{'newLine':'0','tab':6}),
		(PF_TOGGLE , 'reverse', '   Reverse gradient? ', 0,["DON'T USE","YES"],{'newLine':1,'tab':6}),			
		(PF_OPTION,  'precisiongap', 'Precision',1, [x[0] for x in PRECISIONS],{'newLine':'0','tab':6}),	
		(PF_OPTION,  'capStylegap', '   Cap style',0,CAP_STYLES,{'newLine':1,'tab':6}),	
		(PF_OPTION,  'joinStylegap', 'Join style',0, JOIN_STYLES,{'newLine':'0','tab':6}),	
		(PF_SPINNER, 'miterLimitgap','   Miter limit',0,(0,100,1),{'newLine':1,'tab':6}),	

		(PF_OPTION,  "use_mer", "Use 3D effect for:", 0, ['Fill Layer','Stroke Active Brush (if used)','Gradient along path (if used)'],{'newLine':1,'tab':7}),
		(PF_TITLE  , "motion_ls"  , "<b>						*** MOTION BLUR ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':7}),	
		(PF_TOGGLE , "motion" , "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':1,'tab':7}),
		(PF_TOGGLE,  "type_motion", "Motion Type:", 0, ['Inward','Outward'],{'newLine':1,'tab':7}),		
		(PF_SPINNER, "motion_x", "Move blur center by X px.\n(from center of image): ", 0, (-1000, 1000, 1),{'newLine':'1','tab':7}),
		(PF_SPINNER, "motion_y", "Move blur center by Y px.\n(from center of image): ", 0, (-1000, 1000, 1),{'newLine':'1','tab':7}),		
		(PF_SPINNER, "length_m", "Length:", 20, (1, 1000, 1),{'newLine':'1','tab':7}),
		(PF_SPINNER, "motion_intensity", "Intensity:", 6, (0, 6, 1),{'newLine':'1','tab':7}),
		(PF_TITLE  , "extrusion"  , "<b>						*** 3D EXTRUSION ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':7}),			
		(PF_TOGGLE , "run3d" ,   "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':1,'tab':7}),
		(PF_SPINNER, "thick3d",  "Thickness (px):", 5, (1, 100, 1),{'newLine':1,'tab':7}),
		(PF_OPTION,  "direct3d", "Direction:", 1, ['0 (N)','45 (NE)','90 (E)','135 (SE)','180 (S)','225 (SW)','270 (W)','315 (NW)'],{'newLine':1,'tab':7}),

		(PF_TITLE  , "blur_ls"  , "<b>				*** SHADOW ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':8}),		
		(PF_TOGGLE , "blur" , "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':'0','tab':8}),		
		(PF_COLOR,   "color_shadow", "   Shadow\n   color:", (0,0,0),0,{'newLine':'1','tab':8}),		
		(PF_SPINNER, "blurx", "  Blur X (px):", 15, (2, 1000, 1),{'newLine':'0','tab':8}),
		(PF_SPINNER, "blury", "   Blur Y (px):", 15, (2, 1000, 1),{'newLine':'1','tab':8}),
		(PF_SPINNER, "movex", "  Translate X:", 0, (-1000, 1000, 1),{'newLine':'0','tab':8}),
		(PF_SPINNER, "movey", "   Translate Y:", 0, (-1000, 1000, 1),{'newLine':'1','tab':8}),
		(PF_TITLE  , "gegl_ls"  , "<b>				*** GEGL LONG SHADOW ***</b>",(0.0,0.0), None  ,{'cols':10,'iniCol':1,'newLine':1,'tab':8}),	
		(PF_TOGGLE , "run_long_s" , "" , 0 ,["Function inactive\n  *Click to Start*","Function Active\n *Click to Stop*"],{'newLine':1,'tab':8}),
		(PF_OPTION,  "style",     "Style:", 3, ["Finite","Infinite","Fading","Fading (fixed length)"],{'newLine':'0','tab':8}),		
		(PF_SPINNER, "angle_ls",  "Angle: ", 45, (-180, 180, 1),{'newLine':1,'tab':8}),
		(PF_SPINNER, "length_ls", "Length:", 20, (0, 1000, 1),{'newLine':'0','tab':8}),		
		(PF_COLOR,   "color_ls",  "Color:", (100,256,256),0,{'newLine':'1','tab':8}),		
		
		(PF_TITLE  , "help"  , hlp ,(10,10), None           ,{'tab':9}),		
		(PF_TAB    , "ptab"    , "tabs" , 0, ('TEXT','Transform','Bend\nWave','Fill','Groove','Stickerify','Stroke\n  Path',' 3 D ','Shadow', ' Help'),{'newLine':'1','tab':9} ),
	],
	[],
	mrq_box_text,
	menu="<Image>/Filters/Text/",
)

main()	
